# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013-2015 Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

"""Tests for :mod:`bridgedb.proxy`."""

import sure

from twisted.internet import defer
from twisted.internet import reactor
from twisted.trial import unittest

from bridgedb import proxy


EXIT_LIST_0 = """\
11.11.11.11
22.22.22.22
123.45.67.89"""

EXIT_LIST_1 = """\
33.33.33.33
44.44.44.44
55.55.55.55
66.66.66.66
77.77.77.77"""

EXIT_LIST_BAD = """\
foo
bar
baz"""


class MockExitListProtocol(proxy.ExitListProtocol):
    """A mocked version of :class:`~bridgedb.proxy.ExitListProtocol`."""

    def __init__(self):
        proxy.ExitListProtocol.__init__(self)
        self._data = EXIT_LIST_0
        self.script = '/bin/echo'
        print()

    def _log(self, msg):
        print("%s: %s" % (self.__class__.__name__, msg))

    def childConnectionLost(self, childFD):
        self._log("childConnectionLost() called with childFD=%s" % childFD)
        proxy.ExitListProtocol.childConnectionLost(self, childFD)

    def connectionMade(self):
        self._log("connectionMade() called")
        proxy.ExitListProtocol.connectionMade(self)

    def errReceived(self, data):
        self._log("errReceived() called with %s" % data)
        proxy.ExitListProtocol.errReceived(self, data)

    def outReceivedData(self, data):
        self._log("outReceivedData() called with %s" % data)
        proxy.ExitListProtocol.outReceivedData(self, data)

    def outConnectionLost(self):
        self._log("outConnectionLost() called")
        proxy.ExitListProtocol.outConnectionLost(self)

    def parseData(self):
        data = self._data.split('\n')
        for line in data:
            line = line.strip()
            if not line: continue
            if line.startswith('<'): break
            if line.startswith('#'): continue
            ip = proxy.isIPAddress(line)
            if ip:
                self._log("adding IP %s to exitlist..." % ip)
                self.exitlist.add(ip)

    def processEnded(self, reason):
        self._log("processEnded() called with %s" % reason)
        proxy.ExitListProtocol.processEnded(self, reason)

    def processExited(self, reason):
        self._log("processExited() called with %s" % reason)
        proxy.ExitListProtocol.processExited(self, reason)


class ProxySetImplementationTest(unittest.TestCase):

    def setUp(self):
        # We have to put something in it, otherwise self.ps.should.be.ok won't
        # think it's truthy:
        self.ps = proxy.ProxySet(['1.1.1.1'])

    def test_instantiation(self):
        self.ps.should.be.ok
        self.ps.should.have.property('__contains__').being.callable
        self.ps.should.have.property('__hash__').being.callable
        self.ps.should.have.property('__iter__').being.callable
        self.ps.should.have.property('__len__').being.callable
        self.ps.should.have.property('add').being.callable
        self.ps.should.have.property('copy').being.callable
        self.ps.should.have.property('contains').being.callable
        self.ps.should.have.property('discard').being.callable
        self.ps.should.have.property('remove').being.callable

        self.ps.should.have.property('difference').being.callable
        self.ps.should.have.property('issubset').being.callable
        self.ps.should.have.property('issuperset').being.callable
        self.ps.should.have.property('intersection').being.callable
        self.ps.should.have.property('symmetric_difference').being.callable
        self.ps.should.have.property('union').being.callable

    def test_attributes(self):
        self.ps.should.have.property('proxies').being.a(list)
        self.ps.should.have.property('exitRelays').being.a(set)
        self.ps.should.have.property('_proxies').being.a(set)
        self.ps.should.have.property('_proxydict').being.a(dict)


class LoadProxiesFromFileIntegrationTests(unittest.TestCase):
    """Unittests for :class:`~bridgedb.proxy.loadProxiesFromFile()`."""

    def setUp(self):
        self.fn0 = '%s-0' % self.__class__.__name__
        self.fn1 = '%s-1' % self.__class__.__name__
        self.badfile = '%s-badfile' % self.__class__.__name__
        self.writeFiles()

    def writeFiles(self):
        with open(self.fn0, 'w') as fh:
            fh.write(EXIT_LIST_0)
            fh.flush()
        with open(self.fn1, 'w') as fh:
            fh.write(EXIT_LIST_1)
            fh.flush()
        with open(self.badfile, 'w') as fh:
            fh.write(EXIT_LIST_BAD)
            fh.flush()

    def emptyFile(self, filename):
        """We have to do this is a separate method, otherwise Twisted doesn't
        actually do it.
        """
        fh = open(filename, 'w')
        fh.truncate()
        fh.close()

    def test_proxy_loadProxiesFromFile_1_file(self):
        """Test loading proxies from one file."""
        proxies = proxy.loadProxiesFromFile(self.fn0)
        self.assertEqual(len(proxies), 3)

    def test_proxy_loadProxiesFromFile_1_file_missing(self):
        """Test loading proxies from one file that doesn't exist."""
        proxies = proxy.loadProxiesFromFile('%s-missing' % self.__class__.__name__)
        self.assertEqual(len(proxies), 0)

    def test_proxy_loadProxiesFromFile_1_file_and_proxyset(self):
        """Test loading proxies from one file."""
        proxyList = proxy.ProxySet(['1.1.1.1'])
        proxies = proxy.loadProxiesFromFile(self.fn0, proxySet=proxyList)
        self.assertEqual(len(proxies), 3)
        self.assertEqual(len(proxyList), 4)

    def test_proxy_loadProxiesFromFile_2_files_and_proxyset(self):
        """Test loading proxies from two files."""
        proxyList = proxy.ProxySet(['1.1.1.1'])
        proxy.loadProxiesFromFile(self.fn0, proxySet=proxyList)
        proxies = proxy.loadProxiesFromFile(self.fn1, proxySet=proxyList)
        self.assertEqual(len(proxies), 5)
        self.assertEqual(len(proxyList), 9)

    def test_proxy_loadProxiesFromFile_removeStale(self):
        """Test loading proxies from two files and removing the stale ones."""
        proxyList = proxy.ProxySet(['1.1.1.1'])
        self.assertEqual(len(proxyList), 1)
        proxies = proxy.loadProxiesFromFile(self.fn0, proxySet=proxyList)
        self.assertEqual(len(proxies), 3)
        self.assertEqual(len(proxyList), 4)
        proxies = proxy.loadProxiesFromFile(self.fn1, proxySet=proxyList)
        self.assertEqual(len(proxies), 5)
        self.assertEqual(len(proxyList), 9)

        self.emptyFile(self.fn0)
        proxies = proxy.loadProxiesFromFile(self.fn0, proxySet=proxyList,
                                            removeStale=True)
        self.assertEqual(len(proxies), 0)
        self.assertEqual(len(proxyList), 6)

    def test_proxy_loadProxiesFromFile_duplicates(self):
        """Loading proxies from the same file twice shouldn't store
        duplicates.
        """
        proxyList = proxy.ProxySet(['1.1.1.1'])
        proxy.loadProxiesFromFile(self.fn1, proxySet=proxyList)
        self.assertEqual(len(proxyList), 6)
        proxy.loadProxiesFromFile(self.fn1, proxySet=proxyList)
        self.assertEqual(len(proxyList), 6)

    def test_proxy_loadProxiesFromFile_bad_file(self):
        """Loading proxies from a file with invalid IPs in it should do
        nothing.
        """
        proxyList = proxy.ProxySet()
        proxy.loadProxiesFromFile(self.badfile, proxySet=proxyList)
        self.assertEqual(len(proxyList), 0)


class DownloadTorExitsTests(unittest.TestCase):
    """Tests for `~bridgedb.proxy.downloadTorExits()`."""

    def setUp(self):
        self.protocol = MockExitListProtocol
        self.proxyList = proxy.ProxySet()

    def tearDown(self):
        """Cleanup method after each ``test_*`` method runs; removes all
        selectable readers and writers from the reactor.
        """
        reactor.removeAll()

    def test_proxy_downloadTorExits(self):
        def do_test():
            return proxy.downloadTorExits(self.proxyList,
                                          'OurIPWouldGoHere',
                                          protocol=self.protocol)
        d = do_test()


class ProxySetUnittests(unittest.TestCase):
    """Unittests for :class:`~bridgedb.proxy.ProxySet`."""

    def setUp(self):
        self.proxies = EXIT_LIST_1.split('\n')
        self.moarProxies = EXIT_LIST_0.split('\n')

        self.proxyList = proxy.ProxySet()
        for p in self.proxies:
            self.proxyList.add(p)

    def test_ProxySet_init(self):
        """When initialised (after setUp() has run), the ProxySet should
        contain a number of proxies equal to the number we added in the setUp()
        method.
        """
        self.assertEquals(len(self.proxyList), len(self.proxies))

    def test_ProxySet_proxies_getter(self):
        """ProxySet.proxies should list all proxies."""
        self.assertItemsEqual(self.proxyList.proxies, set(self.proxies))

    def test_ProxySet_proxies_setter(self):
        """``ProxySet.proxies = ['foo']`` should raise an ``AttributeError``."""
        self.assertRaises(AttributeError, self.proxyList.__setattr__, 'proxies', ['foo'])

    def test_ProxySet_proxies_deleter(self):
        """``del(ProxySet.proxies)`` should raise an AttributeError."""
        self.assertRaises(AttributeError, self.proxyList.__delattr__, 'proxies')

    def test_ProxySet_exitRelays_issubset_proxies(self):
        """ProxySet.exitRelays should always be a subset of ProxySet.proxies."""
        self.assertTrue(self.proxyList.exitRelays.issubset(self.proxyList.proxies))
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertTrue(self.proxyList.exitRelays.issubset(self.proxyList.proxies))

    def test_ProxySet_exitRelays_getter(self):
        """ProxySet.exitRelays should list all exit relays."""
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertItemsEqual(self.proxyList.exitRelays, set(self.moarProxies))

    def test_ProxySet_exitRelays_setter(self):
        """``ProxySet.exitRelays = ['foo']`` should raise an ``AttributeError``."""
        self.assertRaises(AttributeError, self.proxyList.__setattr__, 'exitRelays', ['foo'])

    def test_ProxySet_exitRelays_deleter(self):
        """``del(ProxySet.exitRelays)`` should raise an AttributeError."""
        self.assertRaises(AttributeError, self.proxyList.__delattr__, 'exitRelays')

    def test_ProxySet_add_new(self):
        """ProxySet.add() should add a new proxy."""
        self.proxyList.add('110.110.110.110')
        self.assertEquals(len(self.proxyList), len(self.proxies) + 1)
        self.assertIn('110.110.110.110', self.proxyList)

    def test_ProxySet_add_None(self):
        """ProxySet.add() called with None should return False."""
        self.assertFalse(self.proxyList.add(None))
        self.assertEquals(len(self.proxyList), len(self.proxies))

    def test_ProxySet_add_duplicate(self):
        """ProxySet.add() shouldn't add the same proxy twice."""
        self.proxyList.add(self.proxies[0])
        self.assertEquals(len(self.proxyList), len(self.proxies))
        self.assertIn(self.proxies[0], self.proxyList)

    def test_ProxySet_addExitRelays(self):
        """ProxySet.addExitRelays() should add the new proxies."""
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertIn(self.moarProxies[0], self.proxyList)

    def test_ProxySet_radd_new(self):
        """ProxySet.radd() should add a new proxy."""
        self.proxyList.__radd__('110.110.110.110')
        self.assertEquals(len(self.proxyList), len(self.proxies) + 1)
        self.assertIn('110.110.110.110', self.proxyList)

    def test_ProxySet_addExitRelays_tagged(self):
        """ProxySet.addExitRelays() should add the new proxies, and they should
        be tagged as being Tor exit relays.
        """
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertTrue(self.proxyList.isExitRelay(self.moarProxies[0]))
        self.assertEquals(self.proxyList.getTag(self.moarProxies[0]),
                          self.proxyList._exitTag)

    def test_ProxySet_addExitRelays_length(self):
        """ProxySet.addExitRelays() should add the new proxies and then the
        total number should be equal to the previous number of proxies plus the
        new exit relays added.
        """
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertEquals(len(self.proxyList), len(self.proxies) + len(self.moarProxies))

    def test_ProxySet_addExitRelays_previous_proxies_kept(self):
        """ProxySet.addExitRelays() should add the new proxies and keep ones that
        were already in the ProxySet.
        """
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertIn(self.proxies[0], self.proxyList)

    def test_ProxySet_addExitRelays_previous_proxies_not_tagged(self):
        """ProxySet.addExitRelays() should add the new proxies and tag them,
        but any previous non-exit relays in the ProxySet shouldn't be tagged as
        being Tor exit relays.
        """
        self.proxyList.addExitRelays(self.moarProxies)
        self.assertFalse(self.proxyList.isExitRelay(self.proxies[0]))
        self.assertNotEquals(self.proxyList.getTag(self.proxies[0]),
                             self.proxyList._exitTag)

    def test_ProxySet_addProxies_tuple_individual_tags(self):
        """ProxySet.addProxies() should add the new proxies and tag them with
        whatever tags we want.
        """
        tags = ['foo', 'bar', 'baz']
        extraProxies = zip(self.moarProxies, tags)
        self.proxyList.addProxies(extraProxies)
        self.assertEquals(len(self.proxyList), len(self.proxies) + len(extraProxies))
        self.assertIn(extraProxies[0][0], self.proxyList)
        self.assertEquals(self.proxyList._proxydict[extraProxies[0][0]], extraProxies[0][1])
        self.assertEquals(self.proxyList._proxydict[extraProxies[1][0]], extraProxies[1][1])
        self.assertEquals(self.proxyList._proxydict[extraProxies[2][0]], extraProxies[2][1])

    def test_ProxySet_addProxies_tuple_too_many_items(self):
        """``ProxySet.addProxies()`` where the tuples have >2 items should
        raise a ValueError.
        """
        extraProxies = zip(self.moarProxies,
                           ['sometag' for _ in range(len(self.moarProxies))],
                           ['othertag' for _ in range(len(self.moarProxies))])
        self.assertRaises(ValueError, self.proxyList.addProxies, extraProxies)

    def test_ProxySet_addProxies_list(self):
        """``ProxySet.addProxies(..., tag='sometag')`` should add the new
        proxies and tag them all with the same tag.
        """
        self.proxyList.addProxies(self.moarProxies, tag='sometag')
        self.assertEquals(len(self.proxyList), len(self.proxies) + len(self.moarProxies))
        self.assertIn(self.moarProxies[0], self.proxyList)
        for p in self.moarProxies:
            self.assertEquals(self.proxyList.getTag(p), 'sometag')
        for p in self.proxies:
            self.assertNotEqual(self.proxyList.getTag(p), 'sometag')

    def test_ProxySet_addProxies_set(self):
        """``ProxySet.addProxies(..., tag=None)`` should add the new
        proxies and tag them all with timestamps.
        """
        self.proxyList.addProxies(set(self.moarProxies))
        self.assertEquals(len(self.proxyList), len(self.proxies) + len(self.moarProxies))
        self.assertIn(self.moarProxies[0], self.proxyList)
        for p in self.moarProxies:
            self.assertIsInstance(self.proxyList.getTag(p), float)
        for p in self.proxies:
            self.assertNotEqual(self.proxyList.getTag(p), 'sometag')

    def test_ProxySet_addProxies_bad_type(self):
        """``ProxySet.addProxies()`` called with something which is neither an
        iterable, a basestring, or an int should raise a ValueError.
        """
        self.assertRaises(ValueError, self.proxyList.addProxies, object)

    def test_ProxySet_addProxies_list_of_bad_types(self):
        """``ProxySet.addProxies()`` called with something which is neither an
        iterable, a basestring, or an int should raise a ValueError.
        """
        self.assertRaises(ValueError, self.proxyList.addProxies, [object, object, object])

    def test_ProxySet_getTag(self):
        """ProxySet.getTag() should get the tag for a proxy in the set."""
        self.proxyList.add('1.1.1.1', 'bestproxyevar')
        self.assertEquals(self.proxyList.getTag('1.1.1.1'), 'bestproxyevar')

    def test_ProxySet_getTag_nonexistent(self):
        """ProxySet.getTag() should get None for a proxy not in the set."""
        self.assertIsNone(self.proxyList.getTag('1.1.1.1'))

    def test_ProxySet_clear(self):
        """ProxySet.clear() should clear the set of proxies."""
        self.proxyList.clear()
        self.assertEquals(len(self.proxyList), 0)
        self.assertEquals(len(self.proxyList.proxies), 0)
        self.assertEquals(len(self.proxyList._proxies), 0)
        self.assertEquals(len(self.proxyList._proxydict.items()), 0)

    def test_ProxySet_contains_list(self):
        """Calling ``list() is in ProxySet()`` should return False."""
        self.assertFalse(self.proxyList.contains(list(self.proxies[0],)))

    def test_ProxySet_contains_nonexistent(self):
        """``ProxySet().contains()`` with a proxy not in the set should
        return False.
        """
        self.assertFalse(self.proxyList.contains(self.moarProxies[0]))

    def test_ProxySet_contains_nonexistent(self):
        """``ProxySet().contains()`` with a proxy in the set should
        return True.
        """
        self.assertTrue(self.proxyList.contains(self.proxies[0]))

    def test_ProxySet_copy(self):
        """ProxySet.copy() should create an exact copy."""
        newProxyList = self.proxyList.copy()
        self.assertEquals(newProxyList, self.proxyList)

    def test_ProxySet_difference(self):
        """ProxySet.difference() should list the items in ProxySetA which
        aren't in ProxySetB.
        """
        proxySetA = self.proxyList
        proxySetB = proxy.ProxySet(self.moarProxies)
        self.assertItemsEqual(proxySetA.difference(proxySetB),
                              set(self.proxies))
        self.assertItemsEqual(proxySetB.difference(proxySetA),
                              set(self.moarProxies))

    def test_ProxySet_firstSeen_returns_timestamp(self):
        """ProxySet.firstSeen() should return a timestamp for a proxy with a
        timestamp tag.
        """
        self.proxyList.add(self.moarProxies[0])
        self.assertIsNotNone(self.proxyList.firstSeen(self.moarProxies[0]))

    def test_ProxySet_firstSeen_returns_float(self):
        """ProxySet.firstSeen() should return a timestamp for a proxy with a
        timestamp tag.
        """
        self.proxyList.add(self.moarProxies[1])
        self.assertIsInstance(self.proxyList.firstSeen(self.moarProxies[1]), float)

    def test_ProxySet_firstSeen_other_tags(self):
        """ProxySet.firstSeen() should return None when a proxy doesn't have a
        timestamp.
        """
        self.proxyList.add(self.moarProxies[2], 'sometag')
        self.assertIsNone(self.proxyList.firstSeen(self.moarProxies[2]))

    def test_ProxySet_issubset(self):
        """ProxySet.issubset() on a superset should return True."""
        self.assertTrue(self.proxyList.issubset(set(self.proxies + self.moarProxies[:0])))

    def test_ProxySet_issuperset(self):
        """ProxySet.issubset() on a subset should return True."""
        self.assertTrue(self.proxyList.issuperset(set(self.proxies[:1])))

    def test_ProxySet_intersection(self):
        """ProxySet.intersection() should return the combination of two
        disjoint sets.
        """
        raise unittest.SkipTest(
            ("FIXME: bridgedb.proxy.ProxySet.intersection() is broken and "
             "always returns an empty set()."))

        a = self.proxies
        a.extend(self.moarProxies)
        a = set(a)
        b = self.proxyList.intersection(set(self.moarProxies))
        self.assertItemsEqual(a, b)

    def test_ProxySet_remove(self):
        """ProxySet.remove() should subtract proxies which were already added
        to the set.
        """
        self.proxyList.remove(self.proxies[0])
        self.assertEquals(len(self.proxyList), len(self.proxies) - 1)
        self.assertNotIn(self.proxies[0], self.proxyList)

    def test_ProxySet_remove_nonexistent(self):
        """ProxySet.remove() shouldn't subtract proxies which aren't already in
        the set.
        """
        self.proxyList.remove('110.110.110.110')
        self.assertEquals(len(self.proxyList), len(self.proxies))
        self.assertNotIn('110.110.110.110', self.proxyList)

    def test_ProxySet_replaceProxyList(self):
        """ProxySet.replaceProxyList should remove all the current proxies and
        add all the new ones.
        """
        self.proxyList.replaceProxyList(self.moarProxies, 'seven proxies')
        for p in self.moarProxies:
            self.assertIn(p, self.proxyList)
            self.assertEqual(self.proxyList.getTag(p), 'seven proxies')
        for p in self.proxies:
            self.assertNotIn(p, self.proxyList)

    def test_ProxySet_replaceProxyList_bad_type(self):
        """ProxySet.replaceProxyList should remove all the current proxies and
        then since we're giving it a bad type it should do nothing else.
        """
        self.proxyList.replaceProxyList([object, object, object])
        self.assertEqual(len(self.proxyList), 0)

    def test_ProxySet_hash(self):
        """Two equal ProxySets should return the same hash."""
        proxyListA = proxy.ProxySet(self.proxies)
        proxyListB = proxy.ProxySet(self.proxies)
        self.assertEqual(proxyListA, proxyListB)
        self.assertItemsEqual(proxyListA, proxyListB)
        self.assertEqual(hash(proxyListA), hash(proxyListB))


class ExitListProtocolTests(unittest.TestCase):
    """Unittests for :class:`~bridgedb.proxy.ExitListProtocol`."""

    def setUp(self):
        self.proto = proxy.ExitListProtocol()

    def test_ExitListProtocol_parseData_error_page(self):
        """ """
        self.proto.data = """\
<!doctype html>
<html lang="en">
<body>
  <div class="content">
  <img src="/torcheck/img/tor-on.png" class="onion" />
  <h4>Welcome to the Tor Bulk Exit List exporting tool.</h4>
  </div>
</body>
</html>"""
        self.proto.parseData()
        self.assertEqual(len(self.proto.exitlist), 0)

    def test_ExitListProtocol_parseData_page_with_3_ips_with_comments(self):
        """ """
        self.proto.data = """\
# This is a list of all Tor exit nodes from the past 16 hours that can contact 1.1.1.1 on port 443 #
# You can update this list by visiting https://check.torproject.org/cgi-bin/TorBulkExitList.py?ip=1.1.1.1&port=443 #
# This file was generated on Fri Feb  6 02:04:27 UTC 2015 #
101.99.64.150
103.10.197.50
103.240.91.7"""
        self.proto.parseData()
        self.assertEqual(len(self.proto.exitlist), 3)

    def test_ExitListProtocol_parseData_page_with_3_ips(self):
        """ """
        self.proto.data = """
101.99.64.150
103.10.197.50
103.240.91.7"""
        self.proto.parseData()
        self.assertEqual(len(self.proto.exitlist), 3)

    def test_ExitListProtocol_parseData_page_with_bad_ip(self):
        """ """
        self.proto.data = """
192.168.0.1
127.0.0.1
103.240.91.7"""
        self.proto.parseData()
        self.assertEqual(len(self.proto.exitlist), 1)



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_strings ; -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

"""Commonly used string constants.

.. todo:: The instructions for the OpenPGP keys in
    :data:`BRIDGEDB_OPENPGP_KEY` are not translated… should we translate them?
    Should we tell users where to obtain GPG4Win/GPGTools/gnupg?  Should those
    instruction be that verbose?  Or should we get rid of the instructions
    altogether, and assume that any encouragement towards using GPG will just
    make users more frustrated, and (possibly) (mis-)direct that frustration
    at Tor or BridgeDB?
"""

from __future__ import unicode_literals

# This won't work on Python2.6, however
#     1) We don't use Python2.6, and
#     2) We don't care about supporting Python2.6, because Python 2.6 (and,
#        honestly, all of Python2) should die.
from collections import OrderedDict


def _(text):
    """This is necessary because strings are translated when they're imported.
    Otherwise this would make it impossible to switch languages more than
    once.

    :returns: The **text**.
    """
    return text


# TRANSLATORS: Please do not translate the word "TYPE".
EMAIL_MISC_TEXT = {
    0: _("""\
[This is an automated message; please do not reply.]"""),
    1: _("""\
Here are your bridges:"""),
    2: _("""\
You have exceeded the rate limit. Please slow down! The minimum time between
emails is %s hours. All further emails during this time period will be ignored."""),
    3: _("""\
COMMANDs: (combine COMMANDs to specify multiple options simultaneously)"""),
    # TRANSLATORS: Please DO NOT translate the word "BridgeDB".
    4: _("Welcome to BridgeDB!"),
    # TRANSLATORS: Please DO NOT translate the words "transport" or "TYPE".
    5: _("Currently supported transport TYPEs:"),
    6: _("Hey, %s!"),
    7: _("Hello, friend!"),
    8: _("Public Keys"),
    # TRANSLATORS: This string will end up saying something like:
    # "This email was generated with rainbows, unicorns, and sparkles
    #  for alice@example.com on Friday, 09 May, 2014 at 18:59:39."
    9: _("""\
This email was generated with rainbows, unicorns, and sparkles
for %s on %s at %s."""),
}

WELCOME = {
    # TRANSLATORS: Please DO NOT translate "BridgeDB".
    # TRANSLATORS: Please DO NOT translate "Pluggable Transports".
    # TRANSLATORS: Please DO NOT translate "Tor".
    # TRANSLATORS: Please DO NOT translate "Tor Network".
    0: _("""\
BridgeDB can provide bridges with several %stypes of Pluggable Transports%s,
which can help obfuscate your connections to the Tor Network, making it more
difficult for anyone watching your internet traffic to determine that you are
using Tor.\n\n"""),

    # TRANSLATORS: Please DO NOT translate "Pluggable Transports".
    1: _("""\
Some bridges with IPv6 addresses are also available, though some Pluggable
Transports aren't IPv6 compatible.\n\n"""),

    # TRANSLATORS: Please DO NOT translate "BridgeDB".
    # TRANSLATORS: The phrase "plain-ol'-vanilla" means "plain, boring,
    # regular, or unexciting". Like vanilla ice cream. It refers to bridges
    # which do not have Pluggable Transports, and only speak the regular,
    # boring Tor protocol. Translate it as you see fit. Have fun with it.
    2: _("""\
Additionally, BridgeDB has plenty of plain-ol'-vanilla bridges %s without any
Pluggable Transports %s which maybe doesn't sound as cool, but they can still
help to circumvent internet censorship in many cases.\n\n"""),
}
"""These strings should go on the first "Welcome" email sent by the
:mod:`~bridgedb.EmailServer`, as well as on the ``index.html`` template used
by the :mod:`~bridgedb.https.server`. They are used as an introduction to
explain what Tor bridges are, what bridges do, and why someone might want to
use bridges.
"""

FAQ = {
    0: _("What are bridges?"),
    1: _("""\
%s Bridges %s are Tor relays that help you circumvent censorship."""),
}

OTHER_DISTRIBUTORS = {
    0: _("I need an alternative way of getting bridges!"),
    1: _("""\
Another way to get bridges is to send an email to %s. Please note that you must
send the email using an address from one of the following email providers:
%s, %s or %s."""),
}

HELP = {
    0: _("My bridges don't work! I need help!"),
    # TRANSLATORS: Please DO NOT translate "Tor".
    1: _("""If your Tor doesn't work, you should email %s."""),
    # TRANSLATORS: Please DO NOT translate "Pluggable Transports".
    # TRANSLATORS: Please DO NOT translate "Tor Browser".
    # TRANSLATORS: Please DO NOT translate "Tor".
    2: _("""\
Try including as much info about your case as you can, including the list of
bridges and Pluggable Transports you tried to use, your Tor Browser version,
and any messages which Tor gave out, etc."""),
}

BRIDGES = {
    0: _("Here are your bridge lines:"),
    1: _("Get Bridges!"),
}

OPTIONS = {
    0: _("Please select options for bridge type:"),
    1: _("Do you need IPv6 addresses?"),
    2: _("Do you need a %s?"),
}

CAPTCHA = {
    0: _('Your browser is not displaying images properly.'),
    1: _('Enter the characters from the image above...'),
}

HOWTO_TBB = {
    0: _("""How to start using your bridges"""),
    # TRANSLATORS: Please DO NOT translate "Tor Browser".
    1: _("""\
To enter bridges into Tor Browser, first go to the %s Tor Browser download
page %s and then follow the instructions there for downloading and starting
Tor Browser."""),
    # TRANSLATORS: Please DO NOT translate "Tor".
    2: _("""\
When the 'Tor Network Settings' dialogue pops up, click 'Configure' and follow
the wizard until it asks:"""),
    # TRANSLATORS: Please DO NOT translate "Tor".
    3: _("""\
Does your Internet Service Provider (ISP) block or otherwise censor connections
to the Tor network?"""),
    # TRANSLATORS: Please DO NOT translate "Tor".
    4: _("""\
Select 'Yes' and then click 'Next'. To configure your new bridges, copy and
paste the bridge lines into the text input box. Finally, click 'Connect', and
you should be good to go! If you experience trouble, try clicking the 'Help'
button in the 'Tor Network Settings' wizard for further assistance."""),
}

EMAIL_COMMANDS = {
    "get help":             _("Displays this message."),
# TRANSLATORS: Please try to make it clear that "vanilla" here refers to the
# same non-Pluggable Transport bridges described above as being
# "plain-ol'-vanilla" bridges.
    "get bridges":          _("Request vanilla bridges."),
    "get ipv6":             _("Request IPv6 bridges."),
    # TRANSLATORS: Please DO NOT translate the word the word "TYPE".
    "get transport [TYPE]": _("Request a Pluggable Transport by TYPE."),
    # TRANSLATORS: Please DO NOT translate "BridgeDB".
    # TRANSLATORS: Please DO NOT translate "GnuPG".
    "get key":              _("Get a copy of BridgeDB's public GnuPG key."),
    #"subscribe":            _("Subscribe to receive new bridges once per week"),
    #"unsubscribe":          _("Cancel a subscription to new bridges"),
}

#-----------------------------------------------------------------------------
#           All of the following containers are untranslated!
#-----------------------------------------------------------------------------

#: SUPPORTED TRANSPORTS is dictionary mapping all Pluggable Transports
#: methodname to whether or not we actively distribute them. The ones which we
#: distribute SHOULD have the following properties:
#:
#:   1. The PT is in a widely accepted, usable state for most Tor users.
#:   2. The PT is currently publicly deployed *en masse*".
#:   3. The PT is included within the transports which Tor Browser offers in
#:      the stable releases.
#:
#: These will be sorted by methodname in alphabetical order.
#:
#: ***Don't change this setting here; change it in :file:`bridgedb.conf`.***
SUPPORTED_TRANSPORTS = {}

#: DEFAULT_TRANSPORT is a string. It should be the PT methodname of the
#: transport which is selected by default (e.g. in the webserver dropdown
#: menu).
#:
#: ***Don't change this setting here; change it in :file:`bridgedb.conf`.***
DEFAULT_TRANSPORT = ''

def _getSupportedTransports():
    """Get the list of currently supported transports.

    :rtype: list
    :returns: A list of strings, one for each supported Pluggable Transport
        methodname, sorted in alphabetical order.
    """
    supported = [name.lower() for name,w00t in SUPPORTED_TRANSPORTS.items() if w00t]
    supported.sort()
    return supported

def _setDefaultTransport(transport):
    global DEFAULT_TRANSPORT
    DEFAULT_TRANSPORT = transport

def _getDefaultTransport():
    return DEFAULT_TRANSPORT

def _setSupportedTransports(transports):
    """Set the list of currently supported transports.

    .. note: You shouldn't need to touch this. This is used by the config file
        parser. You should change the SUPPORTED_TRANSPORTS dictionary in
        :file:`bridgedb.conf`.

    :param dict transports: A mapping of Pluggable Transport methodnames
        (strings) to booleans.  If the boolean is ``True``, then the Pluggable
        Transport is one which we will (more easily) distribute to clients.
        If ``False``, then we (sort of) don't distribute it.
    """
    global SUPPORTED_TRANSPORTS
    SUPPORTED_TRANSPORTS = transports

def _getSupportedAndDefaultTransports():
    """Get a dictionary of currently supported transports, along with a boolean
    marking which transport is the default.

    It is returned as a :class:`collections.OrderedDict`, because if it is a
    regular dict, then the dropdown menu would populated in random order each
    time the page is rendered.  It is sorted in alphabetical order.

    :rtype: :class:`collections.OrderedDict`
    :returns: An :class:`~collections.OrderedDict` of the Pluggable Transport
        methodnames from :data:`SUPPORTED_TRANSPORTS` whose value in
        ``SUPPORTED_TRANSPORTS`` is ``True``.  If :data:`DEFAULT_TRANSPORT` is
        set, then the PT methodname in the ``DEFAULT_TRANSPORT`` setting is
        added to the :class:`~collections.OrderedDict`, with the value
        ``True``. Every other transport in the returned ``OrderedDict`` has
        its value set to ``False``, so that only the one which should be the
        default PT is ``True``.
    """
    supported = _getSupportedTransports()
    transports = OrderedDict(zip(supported, [False for _ in range(len(supported))]))

    if DEFAULT_TRANSPORT:
        transports[DEFAULT_TRANSPORT] = True

    return transports

EMAIL_SPRINTF = {
    # Goes into the "%s types of Pluggable Transports %s" part of ``WELCOME[0]``
    "WELCOME0": ("", "[0]"),
    # Goes into the "%s without Pluggable Transport %s" part of ``WELCOME[2]``
    "WELCOME2": ("-", "-"),
    # For the "%s Tor Browser download page %s" part of ``HOWTO_TBB[1]``
    "HOWTO_TBB1": ("", "[0]"),
    # For the "you should email %s" in ``HELP[0]``
    "HELP0": ("help@rt.torproject.org"),
}
"""``EMAIL_SPRINTF`` is a dictionary that maps translated strings which
contain format specifiers (i.e. ``%s``) to what those format specifiers should
be replaced with in a given template system.

For example, a string which needs a pair of HTML ``("<a href=''">, "</a>")``
tags (for the templates used by :mod:`bridgedb.https.server`) would need some
alternative replacements for the :mod:`EmailServer`, because the latter uses
templates with a ``text/plain`` mimetype instead of HTML. For the
``EmailServer``, the format strings specifiers are replaced with an empty
string where the opening ``<a>`` tags would go, and a numbered Markdown link
specifier where the closing ``</a>`` tags would go.

The keys in this dictionary are the Python variable names of the corresponding
strings which are being formatted, i.e. ``WELCOME0`` would be the string
replacements for ``strings.WELCOME.get(0)``.


For example, the ``0`` string in :data:`WELCOME` above has the substring::

    "%s without Pluggable Transport %s"

and so to replace the two ``%s`` format specifiers, you would use this mapping
like so::

>>> from bridgedb import strings
>>> welcome = strings.WELCOME[0] % strings.EMAIL_SPRINTF["WELCOME0"]
>>> print welcome.split('\n')[0]
BridgeDB can provide bridges with several types of Pluggable Transports[0],

"""

EMAIL_REFERENCE_LINKS = {
    "WELCOME0": "[0]: https://www.torproject.org/docs/pluggable-transports.html",
    "HOWTO_TBB1": "[0]: https://www.torproject.org/projects/torbrowser.html",
}

BRIDGEDB_OPENPGP_KEY = """\
# This keypair contains BridgeDB's online signing and encryption subkeys. This
# keypair rotates because it is kept online. However, the current online
# keypair will *ALWAYS* be certified by the offline keypair (at the bottom of
# this file).
#
# If you receive an email from BridgeDB, it should be signed with the
# 21B554E95938F4D0 subkey from the following keypair:

# pub   4096R/8DC43A2848821E32 2013-09-11 [expires: 2015-09-11]
#       Key fingerprint = DF81 1109 E17C 8BF1 34B5  EEB6 8DC4 3A28 4882 1E32
# uid                          BridgeDB <bridges@bridges.torproject.org>
# sub   4096R/21B554E95938F4D0 2013-09-11 [expires: 2015-09-11]
#       Key fingerprint = 9FE3 9D1A 7438 9223 3B3F  66F2 21B5 54E9 5938 F4D0
# sub   4096R/E7793047C5B54232 2013-09-11 [expires: 2015-09-11]
#       Key fingerprint = CFFB 8469 9048 37E7 8CAE  322C E779 3047 C5B5 4232

-----BEGIN PGP PUBLIC KEY BLOCK-----


9OgcN7A8kFuNpgp2EHqj1/jrwd7mZYKsWTuGiR/7fwXf+4xbvg==
=raCx
-----END PGP PUBLIC KEY BLOCK-----

# The following keypair is BridgeDB's offline certification-only keypair. It
# is used to sign new online signing/encryption keypairs.
#
# If you import this key and mark it as trusted, emails from BridgeDB (if
# signed correctly with the online keypair above) should always be trusted. To
# do this, open a shell and do:
#
#     $ curl -O https://bridges.torproject.org/keys
#     $ gpg --import keys
#     $ gpg --check-sigs 7B78437015E63DF47BB1270ACBD97AA24E8E472E
#     $ gpg --edit-key 7B78437015E63DF47BB1270ACBD97AA24E8E472E
#
# Then type 'trust' to set the trust level. Choose a number that you like.
# Next type 'quit'. Finally, to create a local signature which will will not
# be uploaded to keyservers:
#
#     $ gpg --lsign-key 7B78437015E63DF47BB1270ACBD97AA24E8E472E
#

# pub   16384R/CBD97AA24E8E472E 2013-10-12
#      Key fingerprint = 7B78 4370 15E6 3DF4 7BB1  270A CBD9 7AA2 4E8E 472E
# uid            BridgeDB (Offline ID Key) <bridges@bridges.torproject.org>
-----BEGIN PGP PUBLIC KEY BLOCK-----


ZxOds8F/UJYDSIB97MQFzBo=
=JdC7
-----END PGP PUBLIC KEY BLOCK-----
"""



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013 Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

"""Parsers for HTTP and Email headers.

.. py:module:: bridgedb.parse.headers
    :synopsis: Parsers for HTTP and Email headers.

bridgedb.parse.headers
=======================
::

 parseAcceptLanguage - Parse the contents of a client 'Accept-Language' header
..
"""

def parseAcceptLanguage(header):
    """Parse the contents of a client 'Accept-Language' header.

    Parse the header in the following manner:

    1. If ``header`` is None or an empty string, return an empty list.
    2. Split the ``header`` string on any commas.
    3. Chop of the RFC2616 quality/level suffix. We ignore these, and just
       use the order of the list as the preference order, without any
       parsing of quality/level assignments.
    4. Add a fallback language of the same type if it is missing. For
       example, if we only got ['es-ES', 'de-DE'], add 'es' after 'es-ES'
       and add 'de' after 'de-DE'.
    5. Change all hyphens to underscores.

    :param string header: The contents of an 'Accept-Language' header, i.e. as
        if taken from :api:`twisted.web.server.Request.getHeader`.
    :rtype: list
    :returns: A list of language codes (with and without locales), in order of
        preference.
    """
    langs = []

    if not header:
        return langs

    langHeader = header.split(',')

    for lang in langHeader:
        if lang.find(';') != -1:
            # Chop off the RFC2616 Accept `q=` and `level=` feilds
            code, _ = lang.split(';')
            langs.append(code)
        else:
            langs.append(lang)

    # Add a fallback language of the same type if it is missing.
    langsWithLocales = filter(lambda x: '-' in x, langs)
    langsOnly = map(lambda x: x.split('-')[0], langsWithLocales)
    for only in langsOnly:
        if only not in langs:
            # Add the fallback after the other languages like it:
            insertAfter = filter(lambda x: x.startswith(only),
                                 [x for x in langs])
            if insertAfter:
                placement = langs.index(insertAfter[0]) + 1
                langs.insert(placement, only)
                continue
            # Otherwise just put it at the end
            langs.append(only)

    # Gettext wants underderscores, because that is how it creates the
    # directories under i18n/, not hyphens:
    langs = map(lambda x: x.replace('-', '_'), [x for x in langs])
    return langs



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013-2015 Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: see included LICENSE for information

"""Tests for :mod:`bridgedb.Main`."""

from __future__ import print_function

import base64
import logging
import os
import random
import shutil
import sys

from datetime import datetime
from time import sleep

from twisted.internet.threads import deferToThread
from twisted.trial import unittest

from bridgedb import Main
from bridgedb.parse.options import parseOptions


logging.getLogger().disabled = True


HERE = os.getcwd()
TOPDIR = HERE.rstrip('_trial_temp')
CI_RUNDIR = os.path.join(TOPDIR, 'run')

# A networkstatus descriptor with two invalid ORAddress (127.0.0.1 and ::1)
# and an invalid port number (70000).
NETWORKSTATUS_MALFORMED = '''\
r OracleLunacy LBXW03FIKvo9aXCEYbDdq1BbNtM E0yN8ofiBpg6JHW0iPX5gJ1gKFI 2014-09-05 21:39:24 127.0.0.1 70000 0
a [::1]:70000
s Fast Guard Running Stable Valid
w Bandwidth=2094050
p reject 1-65535
'''

def mockUpdateBridgeHistory(bridges, timestamps):
    """A mocked version of :func:`bridgedb.Stability.updateBridgeHistory`
    which doesn't access the database (so that we can test functions which
    call it, like :func:`bridgedb.Main.load`).
    """
    for fingerprint, stamps in timestamps.items()[:]:
        for timestamp in stamps:
            print("Pretending to update Bridge %s with timestamp %s..." %
                  (fingerprint, timestamp))


class MockHashring(object):
    def __init__(self):
        self._bridges = {}
    def __len__(self):
        return len(self._bridges.keys())
    def insert(self, bridge):
        self._bridges[bridge.fingerprint] = bridge
    def clear(self):
        pass
    def dumpAssignments(self):
        pass


class MainTests(unittest.TestCase):
    """Integration tests for :func:`bridgedb.Main.load`."""

    def _appendToFile(self, file, data):
        """Append **data** to **file**."""
        fh = open(file, 'a')
        fh.write(data)
        fh.flush()
        fh.close()

    def _copyDescFilesHere(self, files):
        """Copy all the **files** to the _trial_tmp/ directory.

        :param list files: A list of strings representing the paths to
            descriptor files. This should probably be taken from a
            ``bridgedb.persistent.Conf`` object which has parsed the
            ``bridgedb.conf`` file in the top-level directory of this repo.
        :rtype: list
        :returns: A list of the new paths (in the ``_trial_tmp`` directory) to
            the copied descriptor files. This should be used to update the
            ``bridgedb.persistent.Conf`` object.
        """
        updatedPaths = []

        for f in files:
            base = os.path.basename(f)
            src = os.path.join(CI_RUNDIR, base)
            if os.path.isfile(src):
                dst = os.path.join(HERE, base)
                shutil.copy(src, dst)
                updatedPaths.append(dst)
            else:
                self.skip = True
                raise unittest.SkipTest(
                    "Can't find mock descriptor files in %s directory" %
                    CI_RUNDIR)

        return updatedPaths

    def _cbAssertFingerprints(self, d):
        """Assert that there are some bridges in the hashring."""
        self.assertGreater(len(self.hashring), 0)
        return d

    def _cbCallUpdateBridgeHistory(self, d, hashring):
        """Fake some timestamps for the bridges in the hashring, and then call
        Main.updateBridgeHistory().
        """
        def timestamp():
            return datetime.fromtimestamp(random.randint(1324285117, 1524285117))

        bridges = hashring._bridges
        timestamps = {}

        for fingerprint, _ in bridges.items():
            timestamps[fingerprint] = [timestamp(), timestamp(), timestamp()]

        return Main.updateBridgeHistory(bridges, timestamps)

    def _eb_Failure(self, failure):
        """If something produces a twisted.python.failure.Failure, fail the
        test with it.
        """
        self.fail(failure)

    def _writeConfig(self, config):
        """Write a config into the current working directory.

        :param str config: A big long multiline string that looks like the
            bridgedb.conf file.
        :rtype: str
        :returns: The pathname of the file that the **config** was written to.
        """
        configFile = os.path.join(os.getcwd(), 'bridgedb.conf')
        fh = open(configFile, 'w')
        fh.write(config)
        fh.flush()
        fh.close()
        return configFile

    def setUp(self):
        """Find the bridgedb.conf file in the top-level directory of this repo,
        copy it and the descriptor files it references to the current working
        directory, produce a state object from the loaded bridgedb.conf file,
        and make an HMAC key.
        """
        # Get the bridgedb.conf file in the top-level directory of this repo:
        self.configFile = os.path.join(TOPDIR, 'bridgedb.conf')
        self.config = Main.loadConfig(self.configFile)

        # Copy the referenced descriptor files from bridgedb/run/ to CWD:
        self.config.STATUS_FILE = self._copyDescFilesHere([self.config.STATUS_FILE])[0]
        self.config.BRIDGE_FILES = self._copyDescFilesHere(self.config.BRIDGE_FILES)
        self.config.EXTRA_INFO_FILES = self._copyDescFilesHere(self.config.EXTRA_INFO_FILES)

        # Initialise the state
        self.state = Main.persistent.State(**self.config.__dict__)
        self.key = base64.b64decode('TvPS1y36BFguBmSOvhChgtXB2Lt+BOw0mGfz9SZe12Y=')

        # Create a pseudo hashring
        self.hashring = MockHashring()

        # Functions which some tests mock, which we'll need to re-replace
        # later in tearDown():
        self._orig_updateBridgeHistory = Main.updateBridgeHistory
        self._orig_sys_argv = sys.argv

    def tearDown(self):
        """Replace the mocked mockUpdateBridgeHistory() function with the
        real function, Stability.updateBridgeHistory().
        """
        Main.updateBridgeHistory = self._orig_updateBridgeHistory
        sys.argv = self._orig_sys_argv

    def test_Main_updateBridgeHistory(self):
        """Main.updateBridgeHistory should update some timestamps for some
        bridges.
        """
        # Mock the updateBridgeHistory() function so that we don't try to
        # access the database:
        Main.updateBridgeHistory = mockUpdateBridgeHistory

        # Get the bridges into the mocked hashring
        d = deferToThread(Main.load, self.state, self.hashring)
        d.addCallback(self._cbAssertFingerprints)
        d.addErrback(self._eb_Failure)
        d.addCallback(self._cbCallUpdateBridgeHistory, self.hashring)
        d.addErrback(self._eb_Failure)
        return d

    def test_Main_load(self):
        """Main.load() should run without error."""
        d = deferToThread(Main.load, self.state, self.hashring)
        d.addCallback(self._cbAssertFingerprints)
        d.addErrback(self._eb_Failure)
        return d

    def test_Main_load_no_state(self):
        """Main.load() should raise SystemExit without a state object."""
        self.assertRaises(SystemExit, Main.load, None, self.hashring)

    def test_Main_load_clear(self):
        """When called with clear=True, load() should run and clear the
        hashrings.
        """
        d = deferToThread(Main.load, self.state, self.hashring, clear=True)
        d.addCallback(self._cbAssertFingerprints)
        d.addErrback(self._eb_Failure)
        return d

    def test_Main_load_collect_timestamps(self):
        """When COLLECT_TIMESTAMPS=True, Main.load() should call
        Main.updateBridgeHistory().
        """
        # Mock the addOrUpdateBridgeHistory() function so that we don't try to
        # access the database:
        Main.updateBridgeHistory = mockUpdateBridgeHistory
        state = self.state
        state.COLLECT_TIMESTAMPS = True

        # The reactor is deferring this to a thread, so the test execution
        # here isn't actually covering the Storage.updateBridgeHistory()
        # function:
        Main.load(state, self.hashring)

    def test_Main_load_malformed_networkstatus(self):
        """When called with a networkstatus file with an invalid descriptor,
        Main.load() should raise a ValueError.
        """
        self._appendToFile(self.state.STATUS_FILE, NETWORKSTATUS_MALFORMED)
        self.assertRaises(ValueError, Main.load, self.state, self.hashring)

    def test_Main_reloadFn(self):
        """Main._reloadFn() should return True."""
        self.assertTrue(Main._reloadFn())

    def test_Main_handleSIGHUP(self):
        """Main._handleSIGHUP() should return True."""
        raise unittest.SkipTest("_handleSIGHUP touches the reactor.")

        self.assertTrue(Main._handleSIGHUP())

    def test_Main_createBridgeRings(self):
        """Main.createBridgeRings() should add three hashrings to the
        hashring.
        """
        proxyList = None
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(self.config,
                                                                  proxyList,
                                                                  self.key)
        # Should have an HTTPSDistributor ring, an EmailDistributor ring,
        # and an UnallocatedHolder ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 3)

    def test_Main_createBridgeRings_with_proxyList(self):
        """Main.createBridgeRings() should add three hashrings to the
        hashring and add the proxyList to the IPBasedDistibutor.
        """
        exitRelays = ['1.1.1.1', '2.2.2.2', '3.3.3.3']
        proxyList = Main.proxy.ProxySet()
        proxyList.addExitRelays(exitRelays)
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(self.config,
                                                                  proxyList,
                                                                  self.key)
        # Should have an HTTPSDistributor ring, an EmailDistributor ring,
        # and an UnallocatedHolder ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 3)
        self.assertGreater(len(httpsDist.proxies), 0)
        self.assertItemsEqual(exitRelays, httpsDist.proxies)

    def test_Main_createBridgeRings_no_https_dist(self):
        """When HTTPS_DIST=False, Main.createBridgeRings() should add only
        two hashrings to the hashring.
        """
        proxyList = Main.proxy.ProxySet()
        config = self.config
        config.HTTPS_DIST = False
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(config,
                                                                  proxyList,
                                                                  self.key)
        # Should have an EmailDistributor ring, and an UnallocatedHolder ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 2)
        self.assertNotIn('https', hashring.rings)
        self.assertNotIn(httpsDist, hashring.ringsByName.values())

    def test_Main_createBridgeRings_no_email_dist(self):
        """When EMAIL_DIST=False, Main.createBridgeRings() should add only
        two hashrings to the hashring.
        """
        proxyList = Main.proxy.ProxySet()
        config = self.config
        config.EMAIL_DIST = False
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(config,
                                                               proxyList,
                                                               self.key)
        # Should have an HTTPSDistributor ring, and an UnallocatedHolder ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 2)
        self.assertNotIn('email', hashring.rings)
        self.assertNotIn(emailDist, hashring.ringsByName.values())

    def test_Main_createBridgeRings_no_reserved_share(self):
        """When RESERVED_SHARE=0, Main.createBridgeRings() should add only
        two hashrings to the hashring.
        """
        proxyList = Main.proxy.ProxySet()
        config = self.config
        config.RESERVED_SHARE = 0
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(config,
                                                                  proxyList,
                                                                  self.key)
        # Should have an HTTPSDistributor ring, and an EmailDistributor ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 2)
        self.assertNotIn('unallocated', hashring.rings)

    def test_Main_createBridgeRings_two_file_buckets(self):
        """When FILE_BUCKETS has two filenames in it, Main.createBridgeRings()
        should add three hashrings to the hashring, then add two
        "pseudo-rings".
        """
        proxyList = Main.proxy.ProxySet()
        config = self.config
        config.FILE_BUCKETS = {
            'bridges-for-support-desk': 10,
            'bridges-for-ooni-tests': 10,
        }
        (hashring, emailDist, httpsDist) = Main.createBridgeRings(config,
                                                                  proxyList,
                                                                  self.key)
        # Should have an HTTPSDistributor ring, an EmailDistributor, and an
        # UnallocatedHolder ring:
        self.assertEqual(len(hashring.ringsByName.keys()), 3)

        # Should have two pseudoRings:
        self.assertEqual(len(hashring.pseudoRings), 2)
        self.assertIn('pseudo_bridges-for-support-desk', hashring.pseudoRings)
        self.assertIn('pseudo_bridges-for-ooni-tests', hashring.pseudoRings)

    def test_Main_run(self):
        """Main.run() should run and then finally raise SystemExit."""
        config = """
BRIDGE_FILES = ["../run/bridge-descriptors"]
EXTRA_INFO_FILES = ["../run/cached-extrainfo", "../run/cached-extrainfo.new"]
STATUS_FILE = "../run/networkstatus-bridges"
HTTPS_CERT_FILE="cert"
HTTPS_KEY_FILE="privkey.pem"
LOGFILE = "bridgedb.log"
PIDFILE = "bridgedb.pid"
DB_FILE = "bridgedist.db"
DB_LOG_FILE = "bridgedist.log"
MASTER_KEY_FILE = "secret_key"
ASSIGNMENTS_FILE = "assignments.log"
LOGLEVEL = "DEBUG"
SAFELOGGING = True
LOGFILE_COUNT = 5
LOGFILE_ROTATE_SIZE = 10000000
LOG_THREADS = False
LOG_TRACE = True
LOG_TIME_FORMAT = "%H:%M:%S"
COLLECT_TIMESTAMPS = False
NO_DISTRIBUTION_COUNTRIES = ['IR', 'SY']
PROXY_LIST_FILES = []
N_IP_CLUSTERS = 3
FORCE_PORTS = [(443, 1)]
FORCE_FLAGS = [("Stable", 1)]
BRIDGE_PURPOSE = "bridge"
TASKS = {'GET_TOR_EXIT_LIST': 3 * 60 * 60,}
SERVER_PUBLIC_FQDN = 'bridges.torproject.org'
SERVER_PUBLIC_EXTERNAL_IP = '38.229.72.19'
HTTPS_DIST = True
HTTPS_BIND_IP = None
HTTPS_PORT = None
HTTPS_N_BRIDGES_PER_ANSWER = 3
HTTPS_INCLUDE_FINGERPRINTS = True
HTTPS_USE_IP_FROM_FORWARDED_HEADER = False
HTTP_UNENCRYPTED_BIND_IP = "127.0.0.1"
HTTP_UNENCRYPTED_PORT = 55555
HTTP_USE_IP_FROM_FORWARDED_HEADER = False
RECAPTCHA_ENABLED = False
RECAPTCHA_PUB_KEY = ''
RECAPTCHA_SEC_KEY = ''
RECAPTCHA_REMOTEIP = ''
GIMP_CAPTCHA_ENABLED = False
GIMP_CAPTCHA_DIR = 'captchas'
GIMP_CAPTCHA_HMAC_KEYFILE = 'captcha_hmac_key'
GIMP_CAPTCHA_RSA_KEYFILE = 'captcha_rsa_key'
EMAIL_DIST = True
EMAIL_FROM_ADDR = "bridges@torproject.org"
EMAIL_SMTP_FROM_ADDR = "bridges@torproject.org"
EMAIL_SMTP_HOST = "127.0.0.1"
EMAIL_SMTP_PORT = 55556
EMAIL_USERNAME = "bridges"
EMAIL_DOMAINS = ["somewhere.com", "somewhereelse.net"]
EMAIL_DOMAIN_MAP = {
    "mail.somewhere.com": "somewhere.com",
    "mail.somewhereelse.net": "somewhereelse.net",
}
EMAIL_DOMAIN_RULES = {
    'somewhere.com': ["ignore_dots", "dkim"],
    'somewhereelse.net': ["dkim"],
}
EMAIL_WHITELIST = {}
EMAIL_BLACKLIST = []
EMAIL_FUZZY_MATCH = 4
EMAIL_RESTRICT_IPS = []
EMAIL_BIND_IP = "127.0.0.1"
EMAIL_PORT = 55557
EMAIL_N_BRIDGES_PER_ANSWER = 3
EMAIL_INCLUDE_FINGERPRINTS = True
EMAIL_GPG_SIGNING_ENABLED = False
EMAIL_GPG_HOMEDIR = '../.gnupg'
EMAIL_GPG_PRIMARY_KEY_FINGERPRINT = '0017098C5DF4197E3C884DCFF1B240D43F148C21'
EMAiL_GPG_PASSPHRASE = None
EMAIL_GPG_PASSPHRASE_FILE = None
HTTPS_SHARE = 10
EMAIL_SHARE = 5
RESERVED_SHARE = 2
FILE_BUCKETS = {}"""
        configFile = self._writeConfig(config)
        
        # Fake some options:
        sys.argv = ['bridgedb', '-r', os.getcwd(), '-c', configFile]
        options = parseOptions()

        self.assertRaises(SystemExit, Main.run, options, reactor=None)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for :class:`bridgedb.parse.descriptors` module."""

from __future__ import print_function

import datetime
import glob
import hashlib
import io
import os
import textwrap

from twisted.trial import unittest
from twisted.trial.unittest import SkipTest

HAS_STEM = False

try:
    from stem.descriptor.server_descriptor import RelayDescriptor
    from stem.descriptor.extrainfo_descriptor import RelayExtraInfoDescriptor
    from stem.descriptor.router_status_entry import RouterStatusEntryV3
    from bridgedb.parse import descriptors
except (ImportError, NameError), error:
    print("There was an error importing stem: %s" % error)
else:
    HAS_STEM = True

from .util import Benchmarker


BRIDGE_NETWORKSTATUS_0 = '''\
r MiserLandfalls 4IsyTSCtChPhFPAnq5rD8yymlqA /GMC4lz8RXT/62v6kZNdmzSmopk 2014-11-04 06:23:22 2.215.61.223 4056 0
a [c5fd:4467:98a7:90be:c76a:b449:8e6f:f0a7]:4055
s Fast Guard Running Stable Valid
w Bandwidth=1678904
p reject 1-65535
'''

BRIDGE_NETWORKSTATUS_1 = '''\
r Unmentionable BgOrX0ViP5hNsK5ZvixAuPZ6EY0 NTg9NoE5ls9KjF96Dp/UdrabZ9Y 2014-11-04 12:23:37 80.44.173.87 51691 0
a [da14:7d1e:ba8e:60d0:b078:3f88:382b:5c70]:51690
s Fast Guard Running Stable Valid
w Bandwidth=24361
p reject 1-65535
'''

BRIDGE_SERVER_DESCRIPTOR = '''\
@purpose bridge
router MiserLandfalls 2.215.61.223 4056 0 0
or-address [c5fd:4467:98a7:90be:c76a:b449:8e6f:f0a7]:4055
platform Tor 0.2.2.39 on Linux
opt protocols Link 1 2 Circuit 1
published 2014-11-04 06:23:22
opt fingerprint E08B 324D 20AD 0A13 E114 F027 AB9A C3F3 2CA6 96A0
uptime 24247659
bandwidth 1977077890 2234957615 1719198165
opt extra-info-digest 1CBBB3D6158F324476E6804B7EE25623899271CB
onion-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAOm4NX2wi8JmgcAyvOyiAEfq9UkzaNHK+VnSZBiPIrb5GAKFibR7S+Bb
7+x7tsT8VBNbe9QmwML2GVah3xXg68gJAksMNIgFdpud+zMhduuGd0jr7V55aLmH
ePGJYCh78B9RqfvmeTridp3pljwcAheKKH/YKi3nv1fPY0BwahurAgMBAAE=
-----END RSA PUBLIC KEY-----
signing-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBANd/JkrTZRT24EkK3DDc/E+Nj1QBnKIm/xXMyW0gkotFOVdewIWjwQ5z
Tn3YbDhrFN0aFYVdVwNbRhW83e+jZDkpIQuxlQOx6bT13vrzmg8ff1tH8I9EePl7
MO4v0DLPIEcu7Zfz90oC1bl36oqNsD4h0v4yK/XjVwLutIGiy3gTAgMBAAE=
-----END RSA PUBLIC KEY-----
contact Somebody <somebody@example.com>
ntor-onion-key NBsk2O6ks5qnxLhhhKPd59zi0IzfjnakoOJP+Cm8OAE
reject *:*
router-signature
-----BEGIN SIGNATURE-----
YYA5wJTHcjqXk/QBaDXHX/4Fb8W2OctF4X4VHyxH9Hsou4Ip7nzdfWzbBTcBiIrt
ybaaMO15L9Ctkli/capN+nCw2jWgivgiPnAmJNmLGeN6skTKjLPAau+839hBuQxu
P2aB/+XQfzFBA5TaWF83coDng4OGodhwHaOx10Kn7Bg=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-11-04 06:23:22
write-history 2014-11-04 06:23:22 (900 s) 3188736,2226176,2866176
read-history 2014-11-04 06:23:22 (900 s) 3891200,2483200,2698240
dirreq-write-history 2014-11-04 06:23:22 (900 s) 1024,0,2048
dirreq-read-history 2014-11-04 06:23:22 (900 s) 0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-11-04 06:23:22 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-11-04 06:23:22 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-11-04 08:10:25
write-history 2014-11-04 08:10:25 (900 s) 3188736,2226176,2866176,2226176
read-history 2014-11-04 08:10:25 (900 s) 3891200,2483200,2698240,2483200
dirreq-write-history 2014-11-04 08:10:25 (900 s) 1024,0,2048,3072
dirreq-read-history 2014-11-04 08:10:25 (900 s) 0,0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-11-04 08:10:25 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-11-04 08:10:25 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-12-04 03:10:25
write-history 2014-12-04 03:10:25 (900 s) 3188736,2226176,2866176,2226176
read-history 2014-12-04 03:10:25 (900 s) 3891200,2483200,2698240,2483200
dirreq-write-history 2014-12-04 03:10:25 (900 s) 1024,0,2048,3072
dirreq-read-history 2014-12-04 03:10:25 (900 s) 0,0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-12-04 03:10:25 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-12-04 03:10:25 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''

BRIDGE_SERVER_DESCRIPTOR_ED25519 = '''\
@purpose bridge
router piratepartei 80.92.79.70 80 0 0
identity-ed25519
-----BEGIN ED25519 CERT-----
AQQABhauAccW2uNOkPPWU7h9x9FFWtUJXCnw423dKqL/89pTHFRcAQAgBADfGmFI
//1tBiZZxZ2aXNvvLbEdS/0XHYCWY6Oz3lHCU2xHCJzW03U7htLpq95lWStr2bMm
D9N1MJp8Zufal71nFV5dgCm0DvMoeCN0d1F6zYnrGvyq+2E6p32x/DG33Qs=
-----END ED25519 CERT-----
master-key-ed25519 3xphSP/9bQYmWcWdmlzb7y2xHUv9Fx2AlmOjs95RwlM
platform Tor 0.2.7.1-alpha-dev on Linux
protocols Link 1 2 Circuit 1
published 2015-06-09 21:59:40
fingerprint 312D 6427 4C29 1560 0584 3EEC B19C 6865 FA3C C10C
uptime 0
bandwidth 14971520 104857600 64512
extra-info-digest 30E10A35CCEA6AA1E04C15FD5F99022F4CACEBC6 pph/KzxlcGa20Sl6/nQl7noyKctzcWkRkTbBX7aIapQ
onion-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBALRXlkmBc96bz/WFSJ0/NoNYuOivpRBkMDqE0617x63EE9zA+BQGVk81
5mbF50IQRS12J3F7x+m7USGF7xcUw+id9pe1jzyyqOTo2BPf2Wemif+CvVc9uD0v
BLO38iImiret0yZtxq3RQ2KaCg2z0y+RPDudR6z/d6V3ASFSlPgBAgMBAAE=
-----END RSA PUBLIC KEY-----
signing-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBALGE2wcWNpWczHlLOa3MbRMKYGDMNe3MsTDKqxftImHuUdMV758q5/4c
2d0znZ1k5zma7TIKXM1xblVWaHmSZ65jMyy0jgZl7SNbxibP3xM8mfHAJOoWfnQu
LSj8tKSir2BdA8rncajrDmtQe0C8mxA/RgUHuB6ZF42kAB9lm/33AgMBAAE=
-----END RSA PUBLIC KEY-----
onion-key-crosscert
-----BEGIN CROSSCERT-----
EPpvZluK8YLLXU00HVskixVqpJfkCeKWXkQPv5Vq87n7E/gtzrVM9A0DasSPHgor
0Y1jP2K/6G0nuloeDZuNNqPxxz7LEKom5q66UO0Tk4Xdnmj1yp/hSsqi/8sUGe9R
BmZmuz45UJGmADiYwwFnwec/bKkX3al4BwuQRHwcZd0=
-----END CROSSCERT-----
ntor-onion-key-crosscert 1
-----BEGIN ED25519 CERT-----
AQoABhSGAd8aYUj//W0GJlnFnZpc2+8tsR1L/RcdgJZjo7PeUcJTABR4eqhKqYNN
Sgpojtm7C+QRvD3mTk06EEbFly9VrXOaSK4BVxTlsHadm4ti7vdqGHbTWN7DRRu6
nnUKJPMOsAk=
-----END ED25519 CERT-----
hidden-service-dir
contact 0x02225522 Frenn vun der Enn (FVDE) <info AT enn DOT lu>
ntor-onion-key ycFwQVUCqJlPaLwJNvlrpgNLwkU780t4pKiILLWZw0o=
reject *:*
router-sig-ed25519 /uWcpQeWcwywFwy+O1WGfLQFuxkLMsy8u+rTTum4CQd8uN7bt3VCHRG82X9sc18rMv2VHUs7b+WZcfX39ADMDw
router-signature
-----BEGIN SIGNATURE-----
FpF1a2jF1gkVbSUEuuDrw8ggeyQl4HLqHGXJM/J3SPQDky0OhvqPEV8E0CpONG38
YNumnkSJ0vjI0YUuVyOZKpODHS/dlXnz5F/Yz8vwQfC7IsNRQgNgf5tbT3iAF8yh
VC4FdHgFlAkXbiqkpWtD0ojJJjLlEeXbmGILjC1Ls2I=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR_ED25519 = '''\
extra-info piratepartei 312D64274C29156005843EECB19C6865FA3CC10C
identity-ed25519
-----BEGIN ED25519 CERT-----
AQQABhauAccW2uNOkPPWU7h9x9FFWtUJXCnw423dKqL/89pTHFRcAQAgBADfGmFI
//1tBiZZxZ2aXNvvLbEdS/0XHYCWY6Oz3lHCU2xHCJzW03U7htLpq95lWStr2bMm
D9N1MJp8Zufal71nFV5dgCm0DvMoeCN0d1F6zYnrGvyq+2E6p32x/DG33Qs=
-----END ED25519 CERT-----
published 2015-06-09 21:59:40
write-history 2015-06-09 19:41:54 (14400 s) 1093632,3138560,1309696,1641472,1064960,1799168
read-history 2015-06-09 19:41:54 (14400 s) 4406272,6537216,5197824,5701632,5342208,5817344
dirreq-write-history 2015-06-09 19:17:22 (14400 s) 28672,1727488,575488,589824,43008,618496
dirreq-read-history 2015-06-09 19:17:22 (14400 s) 0,0,0,0,0,0
geoip-db-digest 0A1F9C09E08F6F2490E8880664D4E863D1680A12
geoip6-db-digest A6E9B5DE6F887315749B29F9C9F698215BE5240A
dirreq-stats-end 2015-06-09 12:33:11 (86400 s)
dirreq-v3-ips ir=8,us=8
dirreq-v3-reqs ir=8,us=8
dirreq-v3-resp ok=8,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=8,timeout=4,running=0
transport scramblesuit 80.92.79.70:7333 password=S3DVRHWD5375I3AA5NMQBG4WED5MBIYD
transport fte 80.92.79.70:7331
transport websocket 80.92.79.70:9901
transport obfs3 80.92.79.70:7332
transport obfs4 80.92.79.70:7334 cert=/Q8QygIhLarhjvB+rKiFvSmXdjhO9AF6OXACR8JH+voMwKF0s5uMaG3H3uEBiZNQI79jPw,iat-mode=0
bridge-stats-end 2015-06-09 12:33:17 (86400 s)
bridge-ips us=24
bridge-ip-versions v4=24,v6=0
bridge-ip-transports obfs4=24
router-sig-ed25519 O+yrUnkHXZ16Cf0+a3gfDl2ggygbxQUal4kRi5BD2v3NW8CrWjqGJLBjked8g5eJCThUXZuraHwkapeu8gtAAg
router-signature
-----BEGIN SIGNATURE-----
HbZs8ckBwKbJ4vg0LJztGosNaDqSRD+pHiWgBAmx9ARbz7niJMY/ql+Qxh7NFifQ
xa39dJvObxE65qeaZJvcznSyEkUDcHBFcHLWZev7XQjXf2no9vUL86JvwBKHHKC1
GnoYumyiqlKn3MOiqVYN5KXhO5i6qN/W8SjMVvywxZI=
-----END SIGNATURE-----
extra-info Unnamed 9673B58C3A72BC279C4FADEA678DEDCF63E524D2
published 2015-06-09 22:00:10
router-signature
-----BEGIN SIGNATURE-----
S57LSzZy2ecjd9jqA5R7nzRUOWJBNVGA8TMLiuWMHXj4DY540ZgbObNtAIU/uzR5
C3sfCVx39iQ39DKgi93zaeZ7s37KGKiUXwJkvDsY0+A2N/TNX5DyI0ZH8WAwCMNq
EXVgdbhn8RrQiVT69evPXjdr6hmgllUofRT7LimvR60=
-----END SIGNATURE-----
'''


class ParseDescriptorsTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.parse.descriptors` module."""

    skip = True if not HAS_STEM else False

    def setUp(self):
        """Test if we have Stem installed. Skip these tests if it's missing."""
        self.expectedIPBridge0 = '2.215.61.223'
        self.expectedIPBridge1 = '80.44.173.87'

        self.expectedFprBridge0 = 'E08B324D20AD0A13E114F027AB9AC3F32CA696A0'

        if self.skip:
            raise unittest.SkipTest("Couldn't import Stem.")

    def writeTestDescriptorsToFile(self, filename, *descriptors):
        """Write **descriptors** to **filename**.

        :param str filename: A filename. It will be appended to the current
            working directory automatically.
        :param str descriptors: Some optional strings containing
            descriptors. Each one will be written to **filename** as-is.
        :rtype: str
        :returns: The full path to the file which was written to.
        """
        descFilename = os.path.join(os.getcwd(), filename)
        with open(descFilename, 'w') as fh:
            for desc in descriptors:
                fh.write(desc)
                fh.flush()
        return descFilename

    def test_parse_descriptors_parseServerDescriptorsFile(self):
        """Test for ``b.p.descriptors.parseServerDescriptorsFile``."""
        descFile = io.BytesIO(BRIDGE_SERVER_DESCRIPTOR)
        routers = descriptors.parseServerDescriptorsFile(descFile)
        self.assertIsInstance(routers, list)
        bridge = routers[0]
        self.assertIsInstance(bridge, RelayDescriptor)
        self.assertEqual(bridge.address, self.expectedIPBridge0)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_return_type(self):
        """``b.p.descriptors.parseNetworkStatusFile`` should return a dict."""
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        self.assertIsInstance(routers, list)

    def test_parse_descriptors_parseNetworkStatusFile_has_RouterStatusEntryV2(self):
        """The items in the dict returned from
        ``b.p.descriptors.parseNetworkStatusFile`` should be
        ``RouterStatusEntryV2``s.
        """
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertIsInstance(bridge, RouterStatusEntryV3)

    def test_parse_descriptors_parseNetworkStatusFile_one_file(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with one bridge
        networkstatus descriptor.
        """
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertEqual(bridge.address, self.expectedIPBridge0)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_two_files(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with two bridge
        networkstatus descriptors.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]

        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0,
                                                   BRIDGE_NETWORKSTATUS_1)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]

        self.assertIn(bridge.address, expectedIPs)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_bad_nickname(self):
        """``b.p.descriptors.parseNetworkStatusFile`` with a bridge
        networkstatus descriptor which has a nickname that is too long should
        raise InvalidRouterNickname.
        """
        unparseable = BRIDGE_NETWORKSTATUS_0.replace(
            'MiserLandfalls',
            'MiserLandfallsWaterfallsSnowfallsAvalanche')
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   unparseable)
        self.assertRaises(descriptors.InvalidRouterNickname,
                          descriptors.parseNetworkStatusFile,
                          descFile)

    def test_parse_descriptors_parseNetworkStatusFile_HSDir_flag(self):
        """A Bridge networkstatus descriptor with the HSDir flag should be
        possible to parse (without errors), however, the flag should be ignored
        (since the :class:`bridgedb.bridges.Flags` class doesn't care about it).

        See also: :trac:`16616`
        """
        unparseable = BRIDGE_NETWORKSTATUS_0.replace(
            's Fast Guard Running Stable Valid',
            's Fast Guard Running Stable Valid HSDir')
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   unparseable)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]

        for flag in [u'Fast', u'Guard', u'Running',
                     u'Stable', u'Valid', u'HSDir']:
            self.assertTrue(flag in bridge.flags,
                            ("Expected to parse the %r flag from a bridge "
                             "networkstatus document, but the flag was not "
                             "found!"))

    def test_parse_descriptors_parseNetworkStatusFile_IPv6_ORAddress(self):
        """A Bridge can't have its primary ORAddress be IPv6 without raising
        a ValueError.
        """
        unparseable = BRIDGE_NETWORKSTATUS_0.replace(
            '2.215.61.223', '[2837:fcd2:387b:e376:34c:1ec7:11ff:1686]')
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   unparseable)
        self.assertRaises(ValueError,
                          descriptors.parseNetworkStatusFile,
                          descFile)

    def test_parse_descriptors_parseNetworkStatusFile_with_annotations(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with some document
        headers before the first 'r'-line.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]
        descFile = 'networkstatus-bridges'

        with open(descFile, 'w') as fh:
            fh.write('signature and stuff from the BridgeAuth would go here\n')
            fh.write('some more annotations with parameters and stuff\n')
            fh.write(BRIDGE_NETWORKSTATUS_0)
            fh.write(BRIDGE_NETWORKSTATUS_1)
            fh.flush()

        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertIn(bridge.address, expectedIPs)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_with_annotations_no_skipping(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with some
        document headers before the first 'r'-line, but without skipping said
        annotations.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]
        descFile = 'networkstatus-bridges'

        with open(descFile, 'w') as fh:
            fh.write('signature and stuff from the BridgeAuth would go here\n')
            fh.write('some more annotations with parameters and stuff\n')
            fh.write(BRIDGE_NETWORKSTATUS_0)
            fh.write(BRIDGE_NETWORKSTATUS_1)
            fh.flush()

        self.assertRaises(ValueError,
                          descriptors.parseNetworkStatusFile,
                          descFile, skipAnnotations=False)

    def test_parse_descriptors_parseExtraInfoFiles_return_type(self):
        """The return type of ``b.p.descriptors.parseExtraInfoFiles``
        should be a dictionary (after deduplication).
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        self.assertIsInstance(routers, dict)

    def test_parse_descriptors_parseExtraInfoFiles_has_BridgeExtraInfoDescriptor(self):
        """The return of ``b.p.descriptors.parseExtraInfoFiles`` should
        contain ``BridgeExtraInfoDescriptor``s.
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        bridge = routers.values()[0]
        self.assertIsInstance(bridge, RelayExtraInfoDescriptor)

    def test_parse_descriptors_parseExtraInfoFiles_one_file(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with only one
        bridge extrainfo file.
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        bridge = routers.values()[0]

        # The number of transports we parsed should be equal to the number of
        # 'transport' lines in the descriptor:
        self.assertEqual(len(bridge.transport),
                         BRIDGE_EXTRA_INFO_DESCRIPTOR.count('transport '))

        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_deduplicate_identical_timestamps(self):
        """Parsing two descriptors for the same bridge with identical
        timestamps should log a ``b.p.descriptors.DescriptorWarning``
        and retain only one copy of the descriptor.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        self.assertEqual(len(routers), 1)

    def test_parse_descriptors_parseExtraInfoFiles_two_files(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with two
        bridge extrainfo files, and check that only the newest extrainfo
        descriptor is used.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        # We shouldn't have duplicates:
        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        # We should only have the newest descriptor:
        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-11-04 08:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_two_files_reverse(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with two bridge
        extrainfo files. This time, they are processed in reverse to ensure
        that we only keep the newer duplicates of descriptors, no matter what
        order they appeared in the files.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-11-04 08:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_three_files(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with three
        bridge extrainfo files, and check that only the newest extrainfo
        descriptor is used.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileThree = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree)

        # We shouldn't have duplicates:
        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        # We should only have the newest descriptor:
        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def createDuplicatesForBenchmark(self, b=1, n=1200):
        """Create a bunch of duplicate extrainfos for benchmark tests.

        :param int b: The number of fake "bridges" to create **n** duplicate
            descriptors for.
        :param int n: The number of duplicate descriptors for each bridge
            **b**.
        """
        descFiles = []

        # The timestamp and fingerprint from BRIDGE_EXTRA_INFO_DESCRIPTOR:
        timestamp  = "2014-11-04 06:23:22"
        Y, M, rest = timestamp.split("-")
        fpr        = "E08B324D20AD0A13E114F027AB9AC3F32CA696A0"
        newerFpr   = "E08B324D20AD0A13E114F027AB9AC3F32CA696A0"

        total = 0
        needed = b * n
        for x in range(b):
            if total >= needed:
                break
            # Re-digest the fingerprint to create a "new" bridge
            newerFpr = hashlib.sha1(newerFpr).hexdigest().upper()
            # Generate n extrainfos with different timestamps:
            count = 0
            for year in range(1, ((n + 1)/ 12) + 2):  # Start from the next year
                if count >= n:
                    break
                for month in range(1, 13):
                    if count < n:
                        newerTimestamp = "-".join([str(int(Y) + year), "%02d" % month, rest])
                        newerDuplicate = BRIDGE_EXTRA_INFO_DESCRIPTOR[:].replace(
                            fpr, newerFpr).replace(
                                timestamp, newerTimestamp)
                        descFiles.append(io.BytesIO(newerDuplicate))
                        count += 1
                        total += 1
                    else:
                        break

        print("Deduplicating %5d total descriptors (%4d per bridge; %3d bridges):"
              % (len(descFiles), n, b), end='\t')
        return descFiles

    def test_parse_descriptors_parseExtraInfoFiles_benchmark_100_bridges(self):
        """Benchmark test for ``b.p.descriptors.parseExtraInfoFiles``."""
        print()
        for i in range(1, 6):
            descFiles = self.createDuplicatesForBenchmark(b=100, n=i)
            with Benchmarker():
                routers = descriptors.parseExtraInfoFiles(*descFiles)

    def test_parse_descriptors_parseExtraInfoFiles_benchmark_1000_bridges(self):
        """Benchmark test for ``b.p.descriptors.parseExtraInfoFiles``."""
        raise SkipTest(("This test can take several minutes to complete. "
                        "Run it on your own free time."))

        print()
        for i in range(1, 6):
            descFiles = self.createDuplicatesForBenchmark(b=1000, n=i)
            with Benchmarker():
                routers = descriptors.parseExtraInfoFiles(*descFiles)

    def test_parse_descriptors_parseExtraInfoFiles_benchmark_10000_bridges(self):
        """Benchmark test for ``b.p.descriptors.parseExtraInfoFiles``.
        The algorithm should grow linearly in the number of duplicates.
        """
        raise SkipTest(("This test can take several minutes to complete. "
                        "Run it on your own free time."))

        print()
        for i in range(1, 6):
            descFiles = self.createDuplicatesForBenchmark(b=10000, n=i)
            with Benchmarker():
                routers = descriptors.parseExtraInfoFiles(*descFiles)

    def test_parse_descriptors_parseExtraInfoFiles_no_validate(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with
        descriptor validation disabled.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  validate=False)
        self.assertGreaterEqual(len(routers), 1)

    def test_parse_descriptors_parseExtraInfoFiles_unparseable(self):
        """Test parsing three extrainfo descriptors: one is a valid descriptor,
        one is an older duplicate, and one is unparseable (it has a bad
        geoip-db-digest line). There should be only one descriptor returned
        after parsing.
        """
        # Give it a bad geoip-db-digest:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F",
                "geoip-db-digest FOOOOOOOOOOOOOOOOOOBAAAAAAAAAAAAAAAAAARR")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileThree = self.writeTestDescriptorsToFile(
            "unparseable-descriptor", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree)
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 1, (
            "There were three extrainfo descriptors: one was a duplicate, "
            "and one was unparseable, so that should only leave one "
            "descriptor remaining."))

        bridge = routers.values()[0]
        self.assertEqual(
            bridge.fingerprint,
            "E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            ("It looks like the (supposedly) unparseable bridge was returned "
             "instead of the valid one!"))
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_unparseable_and_parseable(self):
        """Test parsing four extrainfo descriptors: two are valid descriptors,
        one is an older duplicate of one of the valid descriptors, and one is
        unparseable (it has a line we shouldn't recognise). There should be
        only two descriptors returned after parsing.
        """
        # Mess up the bridge-ip-transports line:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "bridge-ip-transports <OR>=8",
                "bridge-ip-transports <OR>")

        parseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "ImOkWithBeingParsed 2B5DA67FBA13A6449DE625673B7AE9E3AA7DF75F")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileThree = self.writeTestDescriptorsToFile(
            "unparseable-descriptor.new", unparseable)
        descFileFour = io.BytesIO(parseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree,
                                                  descFileFour)
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 2, (
            "There were four extrainfo descriptors: one was a duplicate, "
            "and one was unparseable, so that should only leave two "
            "descriptors remaining."))

        self.assertNotIn("F373CC1D86D82267F1F1F5D39470F0E0A022122E", routers.keys(),
                         "The 'unparseable' descriptor was returned by the parser.")

        self.assertIn("E08B324D20AD0A13E114F027AB9AC3F32CA696A0", routers.keys(),
            ("A bridge extrainfo which had duplicates was completely missing "
             "from the data which the parser returned."))
        self.assertEqual(
            routers["E08B324D20AD0A13E114F027AB9AC3F32CA696A0"].published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

        self.assertIn("2B5DA67FBA13A6449DE625673B7AE9E3AA7DF75F", routers.keys(),
                      "The 'parseable' descriptor wasn't returned by the parser.")

    def test_parse_descriptors_parseExtraInfoFiles_bad_signature_footer(self):
        """Calling parseExtraInfoFiles() with a descriptor which has a
        signature with a bad "-----END SIGNATURE-----" footer should return
        zero parsed descriptors.
        """
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            '-----END SIGNATURE-----',
            '-----END SIGNATURE FOR REALZ-----')
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "bad-signature-footer", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_missing_signature(self):
        """Calling parseExtraInfoFiles() with a descriptor which is
        missing the signature should return zero parsed descriptors.
        """
        # Remove the signature
        BEGIN_SIG = '-----BEGIN SIGNATURE-----'
        unparseable, _ = BRIDGE_EXTRA_INFO_DESCRIPTOR.split(BEGIN_SIG)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "missing-signature", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_bad_signature_too_short(self):
        """Calling _verifyExtraInfoSignature() with a descriptor which has a
        bad signature should raise an InvalidExtraInfoSignature exception.
        """
        # Truncate the signature to 50 bytes
        BEGIN_SIG = '-----BEGIN SIGNATURE-----'
        doc, sig = BRIDGE_EXTRA_INFO_DESCRIPTOR.split(BEGIN_SIG)
        unparseable = BEGIN_SIG.join([doc, sig[:50]])
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "truncated-signature", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_unparseable_BytesIO(self):
        """Test parsing three extrainfo descriptors: one is a valid descriptor,
        one is an older duplicate, and one is unparseable (it has a bad
        geoip-db-digest line). The parsing should raise an unhandled
        AttributeError because _copyUnparseableDescriptorFile() tries to
        manipulate the io.BytesIO object's filename, and it doesn't have one.
        """
        # Give it a bad geoip-db-digest:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F",
                "geoip-db-digest FOOOOOOOOOOOOOOOOOOBAAAAAAAAAAAAAAAAAARR")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        descFileThree = io.BytesIO(unparseable)
        self.assertRaises(AttributeError,
                          descriptors.parseExtraInfoFiles,
                          descFileOne, descFileTwo, descFileThree)

    def test_parse_descriptors_parseExtraInfoFiles_empty_file(self):
        """Test parsing an empty extrainfo descriptors file."""
        routers = descriptors.parseExtraInfoFiles(io.BytesIO(''))
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_ed25519(self):
        """Test parsing an extrainfo descriptor with Ed25519 keys/certificates.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_ED25519)
        routers = descriptors.parseExtraInfoFiles(descFileOne)
        self.assertEqual(len(routers), 1)

    def test_parse_descriptors_parseExtraInfoFiles_ed25519(self):
        """Test parsing an extrainfo descriptor with Ed25519 keys/certificates.
        """
        descFileOne = io.BytesIO(BRIDGE_SERVER_DESCRIPTOR_ED25519)
        routers = descriptors.parseServerDescriptorsFile(descFileOne)
        self.assertIsInstance(routers, list)
        self.assertEqual(len(routers), 1)

        bridge = routers[0]
        self.assertIsInstance(bridge, RelayDescriptor)
        self.assertEqual(bridge.address, u'80.92.79.70')
        self.assertEqual(bridge.fingerprint, u'312D64274C29156005843EECB19C6865FA3CC10C')

    def test_parse_descriptors_copyUnparseableDescriptorFile_return_value(self):
        """``b.p.descriptors._copyUnparseableDescriptorFile()`` should return
        True when the new file is successfully created.
        """
        filename = "bridge-descriptors"
        with open(filename, 'w') as fh:
            fh.write(BRIDGE_SERVER_DESCRIPTOR)
            fh.flush()

        result = descriptors._copyUnparseableDescriptorFile(filename)
        self.assertTrue(result)  # should return True

    def test_parse_descriptors_copyUnparseableDescriptorFile_new_filename(self):
        """``b.p.descriptors._copyUnparseableDescriptorFile()`` should create a
        copy of the bad file with a specific filename format.
        """
        filename = "bridge-descriptors"
        with open(filename, 'w') as fh:
            fh.write(BRIDGE_SERVER_DESCRIPTOR)
            fh.flush()

        descriptors._copyUnparseableDescriptorFile(filename)
        matchingFiles = glob.glob("*_bridge-descriptors.unparseable")
        self.assertEqual(len(matchingFiles), 1)

        newFile = matchingFiles[-1]
        self.assertTrue(os.path.isfile(newFile))

        timestamp = datetime.datetime.strptime(newFile.split("_")[0],
                                               "%Y-%m-%d-%H:%M:%S")
        # The timestamp should be roughly today (unless we just passed
        # midnight, then it might be +/- 1):
        self.assertApproximates(timestamp.now().day, timestamp.day, 1)

        # The timestamp should be roughly this hour (+/- 1):
        self.assertApproximates(timestamp.now().hour, timestamp.hour, 1)



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_filters ; -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Nick Mathewson <nickm@torproject.org>
#           Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2013-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""Functions for filtering :class:`Bridges <bridgedb.bridges.Bridge>`."""

import logging

from ipaddr import IPv4Address
from ipaddr import IPv6Address

from bridgedb.parse.addr import isIPv


_cache = {}


def bySubring(hmac, assigned, total):
    """Create a filter function which filters for only the bridges which fall
    into the same **assigned** subhashring (based on the results of an **hmac**
    function).

    :type hmac: callable
    :param hmac: An HMAC function, i.e. as returned from
        :func:`bridgedb.crypto.getHMACFunc`.
    :param int assigned: The subring number that we wish to draw bridges from.
        For example, if a user is assigned to subring 2of3 based on their IP
        address, then this function should only return bridges which would
        also be assigned to subring 2of3.
    :param int total: The total number of subrings.
    :rtype: callable
    :returns: A filter function for :class:`Bridges <bridgedb.bridges.Bridge>`.
    """
    logging.debug(("Creating a filter for assigning bridges to subhashring "
                   "%s-of-%s...") % (assigned, total))

    name = "-".join([str(hmac("")[:8]).encode('hex'),
                     str(assigned), "of", str(total)])
    try:
        return _cache[name]
    except KeyError:
        def _bySubring(bridge):
            position = int(hmac(bridge.identity)[:8], 16)
            which = (position % total) + 1
            return True if which == assigned else False
        # The `description` attribute must contain an `=`, or else
        # dumpAssignments() will not work correctly.
        setattr(_bySubring, "description", "ring=%d" % assigned)
        _bySubring.__name__ = ("bySubring%sof%s" % (assigned, total))
        _bySubring.name = name
        _cache[name] = _bySubring
        return _bySubring

def byFilters(filtres):
    """Returns a filter which filters by multiple **filtres**.

    :param list filtres: A list (or other iterable) of callables which some
        :class:`Bridges <bridgedb.bridges.Bridge>` should be filtered
        according to.
    :rtype: callable
    :returns: A filter function for :class:`Bridges <bridgedb.bridges.Bridge>`.
    """
    name = []
    for filtre in filtres:
        name.extend(filtre.name.split(" "))
    name = " ".join(set(name))

    try:
        return _cache[name]
    except KeyError:
        def _byFilters(bridge):
            results = [f(bridge) for f in filtres]
            if False in results:
                return False
            return True
        setattr(_byFilters, "description",
                " ".join([getattr(f, "description", "") for f in filtres]))
        _byFilters.name = name
        _cache[name] = _byFilters
        return _byFilters

def byIPv(ipVersion=None):
    """Return ``True`` if at least one of the **bridge**'s addresses has the
    specified **ipVersion**.

    :param int ipVersion: Either ``4`` or ``6``.
    """
    if not ipVersion in (4, 6):
        ipVersion = 4

    name = "ipv%d" % ipVersion
    try:
        return _cache[name]
    except KeyError:
        def _byIPv(bridge):
            """Determine if the **bridge** has an IPv{0} address.

            :type bridge: :class:`bridgedb.bridges.Bridge`
            :param bridge: A bridge to filter.
            :rtype: bool
            :returns: ``True`` if the **bridge** has an address with the
                correct IP version; ``False`` otherwise.
            """
            if isIPv(ipVersion, bridge.address):
                return True
            else:
                for address, port, version in bridge.allVanillaAddresses:
                    if version == ipVersion or isIPv(ipVersion, address):
                        return True
            return False
        setattr(_byIPv, "description", "ip=%d" % ipVersion)
        _byIPv.__name__ = "byIPv%d()" % ipVersion
        _byIPv.func_doc = _byIPv.func_doc.format(ipVersion)
        _byIPv.name = name
        _cache[name] = _byIPv
        return _byIPv

byIPv4 = byIPv(4)
byIPv6 = byIPv(6)

def byTransport(methodname=None, ipVersion=None):
    """Returns a filter function for a :class:`~bridgedb.bridges.Bridge`.

    The returned filter function should be called on a
    :class:`~bridgedb.bridges.Bridge`.  It returns ``True`` if the
    :class:`~bridgedb.bridges.Bridge` has a
    :class:`~bridgedb.bridges.PluggableTransport` such that:

    1. The :data:`methodname <bridgedb.bridges.PluggableTransport.methodname>`
       matches **methodname**, and,

    2. The :attr:`bridgedb.bridges.PluggableTransport.address.version`
       equals the **ipVersion**.

    :param str methodname: A Pluggable Transport
        :data:`~bridgedb.bridges.PluggableTransport.methodname`.
    :param int ipVersion: Either ``4`` or ``6``. The IP version that the
        ``Bridge``'s ``PluggableTransport``
        :attr:`address <bridgedb.bridges.PluggableTransport.address>` should
        have.
    :rtype: callable
    :returns: A filter function for :class:`Bridges <bridgedb.bridges.Bridge>`.
    """
    if not ipVersion in (4, 6):
        ipVersion = 4
    if not methodname:
        return byIPv(ipVersion)

    methodname = methodname.lower()
    name = "transport-%s ipv%d" % (methodname, ipVersion)

    try:
        return _cache[name]
    except KeyError:
        def _byTransport(bridge):
            for transport in bridge.transports:
                if transport.methodname == methodname:
                    if transport.address.version == ipVersion:
                        return True
            return False
        setattr(_byTransport, "description", "transport=%s" % methodname)
        _byTransport.__name__ = "byTransport(%s,%s)" % (methodname, ipVersion)
        _byTransport.name = name
        _cache[name] = _byTransport
        return _byTransport

def byNotBlockedIn(countryCode=None, methodname=None, ipVersion=4):
    """Returns a filter function for :class:`Bridges <bridgedb.bridges.Bridge>`.

    If a Pluggable Transport **methodname** was not specified, the returned
    filter function returns ``True`` if any of the ``Bridge``'s addresses or
    :class:`~bridgedb.bridges.PluggableTransport` addresses aren't blocked in
    **countryCode**.  See :meth:`~bridgedb.bridges.Bridge.isBlockedIn`.

    Otherwise, if a Pluggable Transport **methodname** was specified, it
    returns ``True`` if the :class:`~bridgedb.bridges.Bridge` has a
    :class:`~bridgedb.bridges.PluggableTransport` such that:

    1. The :data:`methodname <bridgedb.bridges.PluggableTransport.methodname>`
       matches **methodname**,

    2. The :attr:`bridgedb.bridges.PluggableTransport.address.version`
       equals the **ipVersion**, and,

    3. The :class:`~bridgedb.bridges.PluggableTransport`.
       :attr:`address <bridgedb.bridges.PluggableTransport.address>` isn't
       known to be blocked in **countryCode**.

    :type countryCode: str or ``None``
    :param countryCode: A two-letter country code which the filtered
        :class:`PluggableTransports <bridgedb.bridges.PluggableTransport>`
        should not be blocked in.
    :param str methodname: A Pluggable Transport
        :data:`methodname <bridgedb.bridges.PluggableTransport.methodname>`.
    :param int ipVersion: Either ``4`` or ``6``. The IP version that the
        ``PluggableTransports``'s addresses should have.
    :rtype: callable
    :returns: A filter function for :class:`Bridges <bridgedb.bridges.Bridge>`.
    """
    if not ipVersion in (4, 6):
        ipVersion = 4
    if not countryCode:
        return byTransport(methodname, ipVersion)

    methodname = methodname.lower() if methodname else methodname
    countryCode = countryCode.lower()

    name = []
    if methodname:
        name.append("transport-%s" % methodname)
    name.append("ipv%d" % ipVersion)
    name.append("not-blocked-in-%s" % countryCode)
    name = " ".join(name)

    try:
        return _cache[name]
    except KeyError:
        def _byNotBlockedIn(bridge):
            if not methodname:
                return not bridge.isBlockedIn(countryCode)
            elif methodname == "vanilla":
                if bridge.address.version == ipVersion:
                    if not bridge.addressIsBlockedIn(countryCode,
                                                     bridge.address,
                                                     bridge.orPort):
                        return True
            else:
                # Since bridge.transportIsBlockedIn() will return True if the
                # bridge has that type of transport AND that transport is
                # blocked, we can "fail fast" here by doing this faster check
                # before iterating over all the transports testing for the
                # other conditions.
                if bridge.transportIsBlockedIn(countryCode, methodname):
                    return False
                else:
                    for transport in bridge.transports:
                        if transport.methodname == methodname:
                            if transport.address.version == ipVersion:
                                return True
            return False
        setattr(_byNotBlockedIn, "description", "unblocked=%s" % countryCode)
        _byNotBlockedIn.__name__ = ("byTransportNotBlockedIn(%s,%s,%s)"
                                    % (methodname, countryCode, ipVersion))
        _byNotBlockedIn.name = name
        _cache[name] = _byNotBlockedIn
        return _byNotBlockedIn



# -*- coding: utf-8 -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________


from twisted.trial import unittest

from bridgedb.bridgerequest import IRequestBridges
from bridgedb.https import request 


class MockRequest(object):
    def __init__(self, args):
        self.args = args


class HTTPSBridgeRequestTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.https.request.HTTPSBridgeRequest`."""

    def setUp(self):
        """Setup test run."""
        self.request = request.HTTPSBridgeRequest()

    def test_HTTPSBridgeRequest_implements_IRequestBridges(self):
        """HTTPSBridgeRequest should implement IRequestBridges interface."""
        self.assertTrue(IRequestBridges.implementedBy(request.HTTPSBridgeRequest))

    def test_HTTPSBridgeRequest_withIPversion(self):
        """HTTPSBridgeRequest.withIPversion({ipv6=[…]}) should store that the
        client wanted IPv6 bridges."""
        parameters = {'ipv6': 'wooooooooo'}
        self.request.withIPversion(parameters)

    def test_HTTPSBridgeRequest_withoutBlockInCountry_IR(self):
        """HTTPSBridgeRequest.withoutBlockInCountry() should add the country CC
        to the ``notBlockedIn`` attribute.
        """
        httprequest = MockRequest({'unblocked': ['ir']})
        self.request.withoutBlockInCountry(httprequest)
        self.assertIn('ir', self.request.notBlockedIn)

    def test_HTTPSBridgeRequest_withoutBlockInCountry_US(self):
        """HTTPSBridgeRequest.withoutBlockInCountry() should add the country CC
        to the ``notBlockedIn`` attribute (and not any other countries).
        """
        httprequest = MockRequest({'unblocked': ['us']})
        self.request.withoutBlockInCountry(httprequest)
        self.assertNotIn('ir', self.request.notBlockedIn)

    def test_HTTPSBridgeRequest_withoutBlockInCountry_no_addClientCountryCode(self):
        """HTTPSBridgeRequest.withoutBlockInCountry(), when
        addClientCountryCode=False, shouldn't add the client's country code to the
        ``notBlockedIn`` attribute.
        """
        httprequest = MockRequest({'unblocked': ['nl']})
        self.request = request.HTTPSBridgeRequest(addClientCountryCode=False)
        self.request.client = '5.5.5.5'
        self.request.withoutBlockInCountry(httprequest)
        self.assertItemsEqual(['nl'], self.request.notBlockedIn)

    def test_HTTPSBridgeRequest_withoutBlockInCountry_bad_params(self):
        """HTTPSBridgeRequest.withoutBlockInCountry() should stop processing if
        the request had a bad "unblocked" parameter.
        """
        httprequest = MockRequest({'unblocked': [3,]})
        self.request.withoutBlockInCountry(httprequest)
        self.assertNotIn('IR', self.request.notBlockedIn)

    def test_HTTPSBridgeRequest_withPluggableTransportType(self):
        """HTTPSBridgeRequest.withPluggableTransportType() should add the
        pluggable transport type to the ``transport`` attribute.
        """
        httprequest = MockRequest({'transport': ['huggable_transport']})
        self.request.withPluggableTransportType(httprequest.args)
        self.assertIn('huggable_transport', self.request.transports)

    def test_HTTPSBridgeRequest_withPluggableTransportType_bad_param(self):
        """HTTPSBridgeRequest.withPluggableTransportType() should stop
        processing if the request had a bad "unblocked" parameter.
        """
        httprequest = MockRequest({'transport': [3,]})
        self.request.withPluggableTransportType(httprequest.args)
        self.assertNotIn('huggable_transport', self.request.transports)



# -*- coding: utf-8  -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""Unittests for :mod:`bridgedb.distribute`."""

from __future__ import print_function

from twisted.trial import unittest

from zope.interface.verify import verifyObject

from bridgedb.distribute import IDistribute
from bridgedb.distribute import Distributor


class DistributorTests(unittest.TestCase):
    """Tests for :class:`bridgedb.distribute.Distributor`."""

    def test_Distributor_implements_IDistribute(self):
        IDistribute.namesAndDescriptions()
        IDistribute.providedBy(Distributor)
        self.assertTrue(verifyObject(IDistribute, Distributor()))

    def test_Distributor_str_no_name(self):
        """str(dist) when the distributor doesn't have a name should return a
        blank string.
        """
        dist = Distributor()
        self.assertEqual(str(dist), "")

    def test_Distributor_str_with_name(self):
        """str(dist) when the distributor has a name should return the name."""
        dist = Distributor()
        dist.name = "foo"
        self.assertEqual(str(dist), "foo")



# -*- coding: utf-8 -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: trygve <tor-dev@lists.torproject.org>
# :copyright: (c) 2014, trygve
#             (c) 2014-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""Integration tests for BridgeDB's HTTPS Distributor.

These tests use `mechanize`_ and `BeautifulSoup`_, and require a BridgeDB
instance to have been started in a separate process. To see how a BridgeDB is
started for our CI infrastructure from a fresh clone of this repository, see
the "before_script" section of the `.travis.yml` file in the top level of this
repository.

.. _mechanize: https://pypi.python.org/pypi/mechanize/
    http://wwwsearch.sourceforge.net/mechanize/
.. _BeautifulSoup:
    http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html
"""

from __future__ import print_function

import gettext
import ipaddr
import mechanize
import os

from BeautifulSoup import BeautifulSoup

from twisted.trial import unittest
from twisted.trial.reporter import TestResult
from twisted.trial.unittest import FailTest
from twisted.trial.unittest import SkipTest

from .test_Tests import DynamicTestCaseMeta
from .util import processExists
from .util import getBridgeDBPID


HTTP_ROOT = 'http://127.0.0.1:6788'
CAPTCHA_RESPONSE = 'Tvx74Pmy'

TOPDIR = os.getcwd()
while not TOPDIR.endswith('bridgedb'):
    TOPDIR = os.path.dirname(TOPDIR)

PIDFILE = os.path.join(TOPDIR, 'run', 'bridgedb.pid')
PID = getBridgeDBPID(PIDFILE)


class HTTPTests(unittest.TestCase):

    def setUp(self):
        if not os.environ.get("CI"):
            raise SkipTest(("The mechanize tests cannot handle self-signed  "
                            "TLS certificates, and thus require opening "
                            "another port for running a plaintext HTTP-only "
                            "BridgeDB webserver. Because of this, these tests "
                            "are only run on CI servers."))
        if not PID or not processExists(PID):
            raise FailTest("Could not start BridgeDB process on CI server!")

        self.br = None

    def tearDown(self):
        self.br = None

    def openBrowser(self):
        # use mechanize to open the BridgeDB website in its browser
        self.br = mechanize.Browser()
        # prevents 'HTTP Error 403: request disallowed by robots.txt'
        self.br.set_handle_robots(False)
        self.br.open(HTTP_ROOT)

        # -------------- Home/Root  page
        self.assertTrue(self.br.viewing_html())
        self.assertEquals(self.br.response().geturl(), HTTP_ROOT)
        self.assertEquals(self.br.title(), "BridgeDB")
        return self.br

    def goToOptionsPage(self):
        # check that we are on the root page
        self.assertTrue(self.br.viewing_html())
        self.assertEquals(self.br.response().geturl(), HTTP_ROOT)

        # follow the link with the word 'bridges' in it.
        # Could also use: text='bridges'
        # Could also use: url='/options'
        self.br.follow_link(text_regex='bridges')

        # ------------- Options
        self.assertEquals(self.br.response().geturl(), HTTP_ROOT + "/options")
        return self.br

    def submitOptions(self, transport, ipv6, captchaResponse):
        # check that we are on the options page
        self.assertEquals(self.br.response().geturl(), HTTP_ROOT + "/options")

        # At this point, we'd like to be able to set some values in
        # the 'advancedOptions' form. Unfortunately the HTML form
        # does not define a 'name' attribute, so the we have to rely on
        # the fact that this is the only form on the page and will therefore
        # always exist at index 0.
        #br.select_form(name="advancedOptions")
        self.br.select_form(nr=0)

        # change the pluggable transport to something else
        self.br.form['transport'] = [transport]
        if ipv6:
            self.br.form['ipv6'] = ['yes']
        self.br.submit()

        # ------------- Captcha
        EXPECTED_URL = HTTP_ROOT + "/bridges?transport=%s" % transport
        if ipv6:
            EXPECTED_URL += "&ipv6=yes"
        self.assertEquals(self.br.response().geturl(), EXPECTED_URL)

        # As on the previous page, the form does not define a 'name'
        # attribute, forcing us to use the index of the form, i.e. 0
        #self.br.select_form(name="captchaSubmission")
        self.br.select_form(nr=0)

        # input the required captcha response. There is only one captcha
        # defined by default, so this should always be accepted. Note this
        # will not be possible to automate if used with a third-party CAPTCHA
        # systems (e.g. reCAPTCHA)
        self.br.form['captcha_response_field'] = captchaResponse
        captcha_response = self.br.submit()

        # ------------- Results
        # URL should be the same as last time
        self.assertEquals(self.br.response().geturl(), EXPECTED_URL)
        soup = BeautifulSoup(captcha_response.read())
        return soup

    def getBridgeLinesFromSoup(self, soup, fieldsPerBridge):
        """We're looking for something like this in the response::
            <div class="bridge-lines">
             obfs2 175.213.252.207:11125 5c6da7d927460317c6ff5420b75c2d0f431f18dd
            </div>
        """
        bridges = []
        soup = soup.findAll(attrs={'class' : 'bridge-lines'})
        self.assertTrue(soup, "Could not find <div class='bridge-lines'>!")

        for portion in soup:
            br_tags = portion.findChildren('br')
            bridge_lines = set(portion.contents).difference(set(br_tags))
            for bridge_line in bridge_lines:
                bridge_line = bridge_line.strip()
                if bridge_line:
                    fields = bridge_line.split()
                    bridges.append(fields)

        self.assertTrue(len(bridges) > 0, "Found no bridge lines in %s" % soup)

        for bridge in bridges:
            self.assertEquals(len(bridge), fieldsPerBridge,
                                  "Expected %d fields in bridge line %s"
                                  % (fieldsPerBridge, bridge))
        return bridges

    def test_content_security_policy(self):
        """Check that the HTTP Content-Security-Policy header is set."""
        self.br = mechanize.Browser()
        self.br.set_handle_robots(False)
        self.br.set_debug_http(True)
        self.br.open(HTTP_ROOT)

        headers = ''.join(self.br.response().info().headers)

        self.assertIn("Content-Security-Policy", headers)
        self.assertIn("default-src 'none';", headers)

    def test_404(self):
        """Asking for a non-existent resource should yield our custom 404 page,
        but we can't actually check because Mechanize flips out if we get
        anything response code other than 200. :/
        """
        page = '/'.join([HTTP_ROOT, '404'])
        self.openBrowser()
        self.assertRaises(mechanize.HTTPError, self.br.open, page)

    def test_get_obfs3_ipv4(self):
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'obfs3'
        soup = self.submitOptions(transport=PT, ipv6=False,
                                  captchaResponse=CAPTCHA_RESPONSE)
        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=3)
        for bridge in bridges:
            pt = bridge[0]
            self.assertEquals(PT, pt)

    def test_get_vanilla_ipv4(self):
        self.openBrowser()
        self.goToOptionsPage()

        PT = '0'
        soup = self.submitOptions(transport=PT, ipv6=False,
                                  captchaResponse=CAPTCHA_RESPONSE)
        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=2)
        for bridge in bridges:
            self.assertTrue(bridge != None)
            addr = bridge[0].rsplit(':', 1)[0]
            self.assertIsInstance(ipaddr.IPAddress(addr), ipaddr.IPv4Address)

    def test_get_vanilla_ipv6(self):
        self.openBrowser()
        self.goToOptionsPage()

        PT = '0'
        soup = self.submitOptions(transport=PT, ipv6=True,
                                  captchaResponse=CAPTCHA_RESPONSE)
        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=2)
        for bridge in bridges:
            self.assertTrue(bridge != None)
            addr = bridge[0].rsplit(':', 1)[0].strip('[]')
            self.assertIsInstance(ipaddr.IPAddress(addr), ipaddr.IPv6Address)

    def test_get_scramblesuit_ipv4(self):
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'scramblesuit'
        soup = self.submitOptions(transport=PT, ipv6=False,
                                  captchaResponse=CAPTCHA_RESPONSE)
        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=4)
        for bridge in bridges:
            pt = bridge[0]
            password = bridge[-1]
            self.assertEquals(PT, pt)
            self.assertTrue(password.find("password=") != -1,
                            "Password field missing expected text")

    def test_get_obfs4_ipv4(self):
        """Try asking for obfs4 bridges, and check that the PT arguments in the
        returned bridge lines were space-separated.

        This is a regression test for #12932, see
        https://bugs.torproject.org/12932.
        """
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'obfs4'

        try:
            soup = self.submitOptions(transport=PT, ipv6=False,
                                      captchaResponse=CAPTCHA_RESPONSE)
        except ValueError as error:
            if 'non-disabled' in str(error):
                raise SkipTest("Pluggable Transport obfs4 is currently disabled.")

        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=6)
        for bridge in bridges:
            pt = bridge[0]
            ptArgs = bridge[-3:]
            self.assertEquals(PT, pt)
            self.assertTrue(len(ptArgs) == 3,
                            ("Expected obfs4 bridge line to have 3 PT args, "
                             "found %d instead: %s") % (len(ptArgs), ptArgs))

    def test_get_obfs4_ipv4_iatmode(self):
        """Ask for obfs4 bridges and check that there is an 'iat-mode' PT
        argument in the bridge lines.
        """
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'obfs4'

        try:
            soup = self.submitOptions(transport=PT, ipv6=False,
                                      captchaResponse=CAPTCHA_RESPONSE)
        except ValueError as error:
            if 'non-disabled' in str(error):
                raise SkipTest("Pluggable Transport obfs4 is currently disabled.")

        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=6)
        for bridge in bridges:
            ptArgs = bridge[-3:]
            hasIATMode = False
            for arg in ptArgs:
                if 'iat-mode' in arg:
                    hasIATMode = True

            self.assertTrue(hasIATMode,
                            "obfs4 bridge line is missing 'iat-mode' PT arg.")

    def test_get_obfs4_ipv4_publickey(self):
        """Ask for obfs4 bridges and check that there is an 'public-key' PT
        argument in the bridge lines.
        """
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'obfs4'

        try:
            soup = self.submitOptions(transport=PT, ipv6=False,
                                      captchaResponse=CAPTCHA_RESPONSE)
        except ValueError as error:
            if 'non-disabled' in str(error):
                raise SkipTest("Pluggable Transport obfs4 is currently disabled.")

        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=6)
        for bridge in bridges:
            ptArgs = bridge[-3:]
            hasPublicKey = False
            for arg in ptArgs:
                if 'public-key' in arg:
                    hasPublicKey = True

            self.assertTrue(hasPublicKey,
                            "obfs4 bridge line is missing 'public-key' PT arg.")

    def test_get_obfs4_ipv4_nodeid(self):
        """Ask for obfs4 bridges and check that there is an 'node-id' PT
        argument in the bridge lines.
        """
        self.openBrowser()
        self.goToOptionsPage()

        PT = 'obfs4'

        try:
            soup = self.submitOptions(transport=PT, ipv6=False,
                                      captchaResponse=CAPTCHA_RESPONSE)
        except ValueError as error:
            if 'non-disabled' in str(error):
                raise SkipTest("Pluggable Transport obfs4 is currently disabled.")

        bridges = self.getBridgeLinesFromSoup(soup, fieldsPerBridge=6)
        for bridge in bridges:
            ptArgs = bridge[-3:]
            hasNodeID = False
            for arg in ptArgs:
                if 'node-id' in arg:
                    hasNodeID = True

            self.assertTrue(hasNodeID,
                            "obfs4 bridge line is missing 'node-id' PT arg.")


class _HTTPTranslationsTests(unittest.TestCase):
    """Build a TestCase with dynamic methods which tests all HTTP rendering of
    all translations in the bridgedb/i18n/ directory.
    """
    i18n = os.path.join(TOPDIR, 'bridgedb', 'i18n')

    def setUp(self):
        if not os.environ.get("CI"):
            raise SkipTest(("The mechanize tests cannot handle self-signed  "
                            "TLS certificates, and thus require opening "
                            "another port for running a plaintext HTTP-only "
                            "BridgeDB webserver. Because of this, these tests "
                            "are only run on CI servers."))

        if not PID or not processExists(PID):
            raise FailTest("Could not start BridgeDB process on CI server!")

        self.br = None

    @classmethod
    def makeTestMethod(cls, locale):
        """Dynamically generate a test_ method for **locale**."""

        def test(self):
            pageArgs = '/?lang=%s' % locale
            language = gettext.translation("bridgedb",
                                           localedir=self.i18n,
                                           languages=[locale,],
                                           fallback=True)
            expected = language.gettext("What are bridges?")

            if not locale.startswith('en'):
                self.assertNotEqual(expected, "What are bridges?")

            self.openBrowser()
            self.br.open(HTTP_ROOT + pageArgs)
            self.assertSubstring(expected, self.br.response().read())

        test.__name__ = 'test_%s' % locale
        setattr(cls, test.__name__, test)

        return test

    def tearDown(self):
        self.br = None

    def openBrowser(self):
        self.br = mechanize.Browser()
        self.br.set_handle_robots(False)

    def test_self(self):
        self.assertTrue(self)


def createHTTPTranslationsTestSuite():
    suite = unittest.TestSuite()
    translations = os.listdir(_HTTPTranslationsTests.i18n)
    translations.remove('templates')

    for locale in translations:
        klass = _HTTPTranslationsTests
        method = klass.makeTestMethod(locale)
        case = klass()
        suite.addTest(case)

    return [suite,]


class HTTPTranslationsTests(unittest.TestCase):
    __metaclass__ = DynamicTestCaseMeta
    testResult    = TestResult()
    testSuites    = createHTTPTranslationsTestSuite()



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_bridgerequest ; -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""API for creating classes which store information on the type of bridges
requested by a client.

.. inheritance-diagram:: BridgeRequestBase
    :parts: 1
"""

import ipaddr
import logging

from zope.interface import implementer
from zope.interface import Attribute
from zope.interface import Interface

from bridgedb.crypto import getHMACFunc
from bridgedb.filters import byIPv
from bridgedb.filters import byNotBlockedIn
from bridgedb.filters import byTransport


class IRequestBridges(Interface):
    """Interface specification of client options for requested bridges."""

    filters = Attribute(
        "A list of callables used to filter bridges from a hashring.")
    ipVersion = Attribute(
        "The IP version of bridge addresses to distribute to the client.")
    transports = Attribute(
        "A list of strings of Pluggable Transport types requested.")
    notBlockedIn = Attribute(
        "A list of two-character country codes. The distributed bridges "
        "should not be blocked in these countries.")
    valid = Attribute(
        "A boolean. Should be ``True`` if the client's request was valid.")
    client = Attribute(
        "This should be some information unique to the client making the "
        "request for bridges, such that we are able to HMAC this unique "
        "data, via :meth:`getHashringPlacement()`, in order to place the "
        "client into a hashring (determining which bridge addresses they get "
        "in the request response).")

    def addFilter():
        """Add a filter to the list of ``filters``."""

    def clearFilters():
        """Clear the list of ``filters``."""

    def generateFilters():
        """Build the list of callables, ``filters``, according to the current
        contents of the lists of ``transports``, ``notBlockedIn``, and the
        ``ipVersion``.
        """

    def getHashringPlacement():
        """Use some unique parameters of the client making this request to
        obtain a value which we can use to place them into one of the hashrings
        with :class:`~bridgedb.bridges.Bridge`s in it, in order to give that
        client different bridges than other clients.
        """

    def isValid():
        """Determine if the request is ``valid`` according to some parameters."""

    def withIPv4():
        """Set the ``ipVersion`` to IPv4."""

    def withIPv6():
        """Set the ``ipVersion`` to IPv6."""

    def withPluggableTransportType(typeOfPT):
        """Add this **typeOfPT** to the list of requested ``transports``."""

    def withoutBlockInCountry(countryCode):
        """Add this **countryCode** to the list of countries which distributed
        bridges should not be blocked in (``notBlockedIn``).
        """


@implementer(IRequestBridges)
class BridgeRequestBase(object):
    """A generic base class for storing options of a client bridge request.

    :vartype filters: list
    :ivar filters: A list of callables used to filter bridges from a hashring.
    :vartype transports: list
    :ivar transports: A list of strings of Pluggable Transport types requested.
    :vartype notBlockedIn: list
    :ivar notBlockedIn: A list of two-character country codes. The distributed
        bridges should not be blocked in these countries.
    :vartype client: str
    :ivar client: This should be some information unique to the client making
        the request for bridges, such that we are able to HMAC this unique
        data in order to place the client into a hashring (determining which
        bridge addresses they get in the request response). It defaults to the
        string ``'default'``.
    :vartype valid: bool
    :ivar valid: Should be ``True`` if the client's request was valid.
    """

    def __init__(self, ipVersion=None):
        self.ipVersion = ipVersion
        self.filters = list()
        self.transports = list()
        self.notBlockedIn = list()
        self.client = 'default'
        self.valid = False

    @property
    def ipVersion(self):
        """The IP version of bridge addresses to distribute to the client.

        :rtype: int
        :returns: Either ``4`` or ``6``.
        """
        return self._ipVersion

    @ipVersion.setter
    def ipVersion(self, ipVersion):
        """The IP version of bridge addresses to distribute to the client.

        :param int ipVersion: The IP address version for the bridge lines we
            should distribute in response to this client request.
        """
        if not ipVersion in (4, 6):
            ipVersion = 4
        self._ipVersion = ipVersion

    def getHashringPlacement(self, key, client=None):
        """Create an HMAC of some **client** info using a **key**.

        :param str key: The key to use for HMACing.
        :param str client: Some (hopefully unique) information about the
            client who is requesting bridges, such as an IP or email address.
        :rtype: long
        :returns: A long specifying index of the first node in a hashring to
            be distributed to the client. This value should obviously be used
            mod the number of nodes in the hashring.
        """
        if not client:
            client = self.client

        # Get an HMAC with the key of the client identifier:
        digest = getHMACFunc(key)(client)
        # Take the lower 8 bytes of the digest and convert to a long:
        position = long(digest[:8], 16)
        return position

    def isValid(self, valid=None):
        """Get or set the validity of this bridge request.

        If called without parameters, this method will return the current
        state, otherwise (if called with the **valid** parameter), it will set
        the current state of validity for this request.

        :param bool valid: If given, set the validity state of this
            request. Otherwise, get the current state.
        """
        if valid is not None:
            self.valid = bool(valid)
        return self.valid

    def withIPv4(self):
        """Set the ``ipVersion`` to IPv4."""
        self.ipVersion = 4

    def withIPv6(self):
        """Set the ``ipVersion`` to IPv6."""
        self.ipVersion = 6

    def withoutBlockInCountry(self, country):
        """Add this **countryCode** to the list of countries which distributed
        bridges should not be blocked in (``notBlockedIn``).
        """
        self.notBlockedIn.append(country.lower())

    def withPluggableTransportType(self, pt):
        """Add this **pt** to the list of requested ``transports``.

        :param str pt: A :class:`~bridgedb.bridges.PluggableTransport`.
            :data:`methodname <bridgedb.bridges.PluggableTransport.methodname>`.
        """
        self.transports.append(pt)

    def addFilter(self, filtre):
        """Add a **filtre** to the list of ``filters``.

        :type filter: callable
        :param filter: A filter function, e.g. one generated via
            :mod:`bridgedb.filters`.
        """
        self.filters.append(filtre)

    def clearFilters(self):
        """Clear the list of ``filters``."""
        self.filters = []

    def justOnePTType(self):
        """Get just one bridge type (e.g. a
        :data:`methodname <bridgedb.bridges.PluggableTransport.methodname>` of
        :class:`~bridgedb.bridges.PluggableTransport`) at a time!
        """
        ptType = None
        try:
            ptType = self.transports[-1]  # Use the last PT requested
        except IndexError:
            logging.debug("No pluggable transports were requested.")
        return ptType

    def generateFilters(self):
        """Build the list of callables, ``filters``, according to the current
        contents of the lists of ``transports``, ``notBlockedIn``, and the
        ``ipVersion``.
        """
        self.clearFilters()

        pt = self.justOnePTType()
        msg = ("Adding a filter to %s for %s for IPv%d"
               % (self.__class__.__name__, self.client, self.ipVersion))

        if self.notBlockedIn:
            for country in self.notBlockedIn:
                logging.info("%s %s bridges not blocked in %s..." %
                             (msg, pt or "vanilla", country))
                self.addFilter(byNotBlockedIn(country, pt, self.ipVersion))
        elif pt:
            logging.info("%s %s bridges..." % (msg, pt))
            self.addFilter(byTransport(pt, self.ipVersion))
        else:
            logging.info("%s bridges..." % msg)
            self.addFilter(byIPv(self.ipVersion))
